import unittest
import uuid
from datetime import datetime, timedelta
import pybrightcove
import mock

# httplib.HTTPConnection
# httplib.HTTPSConnection
# urllib2.urlopen
# tempfile.mkstemp
# ftplib.FTP

class APIConnectionTest(unittest.TestCase):

    def setUp(self):
        self.api = pybrightcove.connection.APIConnection(read_token="read", write_token="write")

    def test_instantiate_valid(self):
        self.assertTrue(hasattr(self.api, "read_token"))
        self.assertTrue(hasattr(self.api, "write_token"))
        self.assertTrue(hasattr(self.api, "read_url"))
        self.assertTrue(hasattr(self.api, "write_url"))
        self.assertEquals(self.api.read_token, "read")
        self.assertEquals(self.api.write_token, "write")
        self.assertTrue(self.api.read_url.startswith('http'))
        self.assertTrue(self.api.write_url.startswith('http'))

    def test_instantiate_invalid(self):
        try:
            c = pybrightcove.connection.APIConnection()
            self.fail("pybrightcove.exceptions.ImproperlyConfiguredError expected")
        except pybrightcove.exceptions.ImproperlyConfiguredError:
            pass

    @mock.patch('urllib2.urlopen')
    def test_post(self, UrlOpenMock):
        u = UrlOpenMock()
        u.read.return_value = '{"result": {"status": "OK"}}'
        d = self.api.post('get_upload_status', video_id=1000)
        self.assertEquals(d, {"status": "OK"})

    @mock.patch('httplib.HTTPConnection')
    @mock.patch('os.fstat')
    @mock.patch("__builtin__.file")
    def test_post_file(self, FileMock, FDStatMock, HTTPMock):
        h = HTTPMock()
        h.getresponse.return_value.read.return_value = '{"result": {"video": 1}}'
        FDStatMock.return_value = [0, 1, 2, 3, 4, 5, 1000]
        f = FileMock()
        f.name = 'bears.mov'
        f.read.return_value = ''
        video = pybrightcove.video.Video(filename='bears.mov', name='The Bears',
            short_description='Opening roll for an exciting soccer match.',
            _connection=self.api)
        video.tags.append('unittest')
        vid = self.api.post('create_video', "bears.mov",
                create_multiple_renditions=True,
                preserve_source_rendition=True,
                encode_to=pybrightcove.enums.EncodeToEnum.FLV,
                video=video._to_dict())
        self.assertEquals(vid, {"video": 1})

    @mock.patch('urllib2.urlopen')
    def test_get_list(self, UrlOpenMock):
        u = UrlOpenMock()
        u.read.return_value = '{"result": {"status": "OK"}}'
        d = self.api.get_item('get_video', video_id=1000)
        self.assertEquals(d, {"result": {"status": "OK"}})

    @mock.patch('urllib2.urlopen')
    def test_get_item(self, UrlOpenMock):
        u = UrlOpenMock()
        
        u.read.return_value = '{"total_count": 5, "page_number": 2, "page_size": 10, "items":[]}'
        d = self.api.get_list('whatever_list', pybrightcove.video.Video, 10, 2, pybrightcove.enums.DEFAULT_SORT_BY, pybrightcove.enums.DEFAULT_SORT_ORDER)
        self.assertEquals(d.data, {"total_count": 5, "page_number": 2, "page_size": 10, "items":[]})
        self.assertEquals(d.page_size, 10)
        self.assertEquals(d.total_count, 5)
        self.assertEquals(d.page_number, 2)




# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
pybrightcove is a simple wrapper around the Brightcove Media API
"""

VERSION = (1, 3, 3, "a", 1) # , "a", 1) # following PEP 386
DEV_N = 1


def get_version():
    version = "%s.%s" % (VERSION[0], VERSION[1])
    if VERSION[2]:
        version = "%s.%s" % (version, VERSION[2])
    if VERSION[3] != "f":
        version = "%s%s%s" % (version, VERSION[3], VERSION[4])
        if DEV_N:
            version = "%s.dev%s" % (version, DEV_N)
    return version


__version__ = get_version()






# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# pylint: disable=C0103,R0913,R0903,R0915,R0902,R0914,R0904,R0201

"""
The ``pybrightcove.video`` module supports all the Brightcove Video APIs.
"""

import hashlib
import os
import time

from datetime import datetime

from pybrightcove import connection
from pybrightcove import enums
from pybrightcove import exceptions


def is_ftp_connection(con):
    """
    Shortcut method to test a connection for if it is an FTP connection or not.
    """
    return isinstance(con, connection.FTPConnection)

def _convert_tstamp(val):
    """
    Safe convert string timestamp into a ``datetime`` object.  Returns None if
    it can't be converted.
    """
    if val:
        return datetime.fromtimestamp(float(val) / 1000)


def _make_tstamp(val):
    """
    Converts a ``datetime`` object into a unix timestamp.
    """
    if val:
        return int(time.mktime(val.timetuple()) * 1000)


class Image(object):
    """
    This object represents metadata about an image file in your account. Images
    are associated with videos as thumbnail images or video still images. An
    image can be a JPEG, GIF, or PNG-formatted image. Note that when creating a
    new image asset, the only property that is required is type. If you are not
    uploading a file, the remoteUrl property is also required.

    For more information, see Adding Images to Videos with the Media API [1].

    [1] http://help.brightcove.com/developer/docs/mediaapi/add_image.cfm
    """

    def __init__(self, data=None, **kwargs):
        self.id = kwargs.get('id', None)
        self.reference_id = kwargs.get('reference_id', None)
        self.type = kwargs.get('type', None)
        self.remote_url = kwargs.get('remote_url', None)
        self.display_name = kwargs.get('display_name', None)

        if data:
            self.id = data['id']
            self.reference_id = data['referenceId']
            self.type = data['type']
            self.remote_url = data["remoteUrl"]
            self.display_name = data["displayName"]

    def to_dict(self):
        """
        Converts object into a dictionary.
        """
        data = {
            'id': self.id,
            'referenceId': self.reference_id,
            'type': self.type,
            'displayName': self.display_name,
            'remoteUrl': self.remote_url}
        for key in data.keys():
            if data[key] == None:
                data.pop(key)
        return data


class Rendition(object):
    """
    The Rendition object represents one of the dynamic delivery renditions of a
    video. A Video should have not more than 10 Rendition objects [1].

    For more information, see Using dynamic delivery [2] and Creating videos
    for dynamic delivery [3].

    [1] http://support.brightcove.com/en/docs/media-api-objects-reference#Rendition
    [2] http://help.brightcove.com/publisher/docs/media/mbr.cfm
    [3] http://help.brightcove.com/developer/docs/mediaapi/create-mbr.cfm
    """

    def __init__(self, data=None):
        self.url = None
        self.encoding_rate = None
        self.frame_height = None
        self.frame_width = None
        self.size = None
        self.remote_url = None
        self.remote_stream_name = None
        self.video_duration = None
        self.video_codec = None

        if data:
            self.url = data.get('url', None)
            self.encoding_rate = data.get('encodingRate', None)
            self.frame_height = data.get('frameHeight', None)
            self.frame_width = data.get('frameWidth', None)
            self.size = data['size']
            self.remote_url = data['remoteUrl']
            self.remote_stream_name = data.get('remoteStreamName', None)
            self.video_duration = data['videoDuration']
            self.video_codec = data['videoCodec']

    def __setattr__(self, name, value):
        msg = None
        if value:
            if name == 'video_duration' and not isinstance(value, (int, long)):
                msg = "Rendition.video_duration must be the duration in "
                msg += "milliseconds as an integer or long."
            if name == 'size' and not isinstance(value, (int, long)):
                msg = "Rendition.size must be the number of bytes as an "
                msg += "integer or long."
            if name == 'video_codec' and \
                    value not in (enums.VideoCodecEnum.SORENSON,
                                  enums.VideoCodecEnum.ON2,
                                  enums.VideoCodecEnum.H264):
                msg = "Rendition.video_codec must be SORENSON, ON2, or H264."

            if msg:
                raise exceptions.PyBrightcoveError(msg)
        return super(Rendition, self).__setattr__(name, value)

    def to_dict(self):
        """
        Converts object into a dictionary.
        """
        data = {
            'url': self.url,
            'encodingRate': self.encoding_rate,
            'frameHeight': self.frame_height,
            'frameWidth': self.frame_width,
            'size': self.size,
            'remoteUrl': self.remote_url,
            'remoteStream': self.remote_stream_name,
            'videoDuration': self.video_duration,
            'videoCodec': self.video_codec}
        [data.pop(key) for key in data.keys() if data[key] is None]
        return data


class CuePoint(object):
    """
    The CuePoint object is a marker set at a precise time point in the duration
    of a video. You can use cue points to trigger mid-roll ads or to separate
    chapters or scenes in a long-form video.

    For more information, see Setting CuePoints with the Media API [1].

    [1] http://help.brightcove.com/developer/docs/mediaapi/cue-points.cfm
    """

    def __init__(self, data=None):
        self.name = None
        self.video_id = None
        self.time = None
        self.force_stop = None
        self.type = None
        self.metadata = None

        if data:
            self.name = data['name']
            self.video_id = data['video_id']
            self.time = data['time']
            self.force_stop = data['forceStop']
            self.type = data['type']
            self.metadata = data['metadata']

    def to_dict(self):
        """
        Converts object into a dictionary.
        """
        data = {
            'name': self.name,
            'video_id': self.video_id,
            'time': self.time,
            'forceStop': self.force_stop,
            'type': self.type,
            'metadata': self.metadata}
        for key in data.keys():
            if data[key] == None:
                data.pop(key)
        return data


class Video(object):
    """
    The Video object is an aggregation of metadata and asset information
    associated with a video.
    """

    # pylint: disable=W0622
    def __init__(self, filename=None, name=None, short_description=None,
        id=None, reference_id=None, renditions=None, data=None,
        _connection=None):

        self._filename = None
        self.name = None
        self.short_description = None
        self.id = None
        self.reference_id = None
        self.accountId = None
        self.long_description = None
        self.flv_url = None
        self.renditions = []
        self.assets = []
        self.metadata = []
        self.video_full_length = None
        self.creation_date = None
        self.published_date = None
        self.last_modified_date = None
        self.item_state = None
        self.start_date = None
        self.end_date = None
        self.link_url = None
        self.link_text = None
        self.tags = []
        self.video_still_url = None
        self.thumbnail_url = None
        self.length = None
        self.economics = None
        self.geo_filtered = None
        self.geo_filtered_countries = None
        self.geo_filtered_exclude = None
        self.cue_points = None
        self.plays_total = None
        self.plays_trailing_week = None

        self.image = None
        self.raw_data = None

        self.connection = _connection
        if not self.connection:
            self.connection = connection.APIConnection()

        if is_ftp_connection(self.connection):
            if reference_id and name and short_description:
                self.reference_id = reference_id
                self.name = name
                self.short_description = short_description
            else:
                msg = "Invalid parameters for Video."
                raise exceptions.PyBrightcoveError(msg)
        else:
            if ((renditions and len(renditions) > 0) or filename) and name \
                and short_description:
                if filename is not None:
                    self._filename = filename
                if renditions is not None:
                    self.renditions = renditions
                self.name = name
                self.short_description = short_description
            elif id or reference_id:
                self.id = id
                self.reference_id = reference_id
                self._find_video()
            elif data:
                self._load(data)
            else:
                msg = "Invalid parameters for Video."
                raise exceptions.PyBrightcoveError(msg)

    def _find_video(self):
        """
        Lookup and populate ``pybrightcove.video.Video`` object given a video
        id or reference_id.
        """
        data = None
        if self.id:
            data = self.connection.get_item(
                'find_video_by_id', video_id=self.id)
        elif self.reference_id:
            data = self.connection.get_item(
                'find_video_by_reference_id', reference_id=self.reference_id)

        if data:
            self._load(data)

    def _to_dict(self):
        """
        Converts object into a dictionary.
        """
        for i, tag in enumerate(self.tags):
            if tag in ("", None):
                self.tags.pop(i)

        data = {
            'name': self.name,
            'referenceId': self.reference_id,
            'shortDescription': self.short_description,
            'longDescription': self.long_description,
            'itemState': self.item_state,
            'linkURL': self.link_url,
            'linkText': self.link_text,
            'tags': self.tags,
            'economics': self.economics,
            'id': self.id,
            'end_date': _make_tstamp(self.end_date),
            'start_date': _make_tstamp(self.start_date)}
        if len(self.renditions) > 0:
            data['renditions'] = []
            for r in self.renditions:
                data['renditions'].append(r.to_dict())
        if len(self.metadata) > 0:
            data['customFields'] = {}
            for meta in self.metadata:
                data['customFields'][meta['key']] = meta['value']
        [data.pop(key) for key in data.keys() if data[key] == None]
        return data

    def to_xml(self):
        # pylint: disable=R0912
        """
        Converts object into an XML string.
        """
        xml = ''
        for asset in self.assets:
            xml += '<asset filename="%s" ' % \
                os.path.basename(asset['filename'])
            xml += ' refid="%(refid)s"' % asset
            xml += ' size="%(size)s"' % asset
            xml += ' hash-code="%s"' % asset['hash-code']
            xml += ' type="%(type)s"' % asset
            if asset.get('encoding-rate', None):
                xml += ' encoding-rate="%s"' % asset['encoding-rate']
            if asset.get('frame-width', None):
                xml += ' frame-width="%s"' % asset['frame-width']
            if asset.get('frame-height', None):
                xml += ' frame-height="%s"' % asset['frame-height']
            if asset.get('display-name', None):
                xml += ' display-name="%s"' % asset['display-name']
            if asset.get('encode-to', None):
                xml += ' encode-to="%s"' % asset['encode-to']
            if asset.get('encode-multiple', None):
                xml += ' encode-multiple="%s"' % asset['encode-multiple']
            if asset.get('h264-preserve-as-rendition', None):
                xml += ' h264-preserve-as-rendition="%s"' % \
                    asset['h264-preserve-as-rendition']
            if asset.get('h264-no-processing', None):
                xml += ' h264-no-processing="%s"' % asset['h264-no-processing']
            xml += ' />\n'
        xml += '<title name="%(name)s" refid="%(referenceId)s" active="TRUE" '
        if self.start_date:
            xml += 'start-date="%(start_date)s" '
        if self.end_date:
            xml += 'end-date="%(end_date)s" '
        for asset in self.assets:
            if asset.get('encoding-rate', None) == None:
                choice = enums.AssetTypeEnum
                if asset.get('type', None) == choice.VIDEO_FULL:
                    xml += 'video-full-refid="%s" ' % asset.get('refid')
                if asset.get('type', None) == choice.THUMBNAIL:
                    xml += 'thumbnail-refid="%s" ' % asset.get('refid')
                if asset.get('type', None) == choice.VIDEO_STILL:
                    xml += 'video-still-refid="%s" ' % asset.get('refid')
                if asset.get('type', None) == choice.FLV_BUMPER:
                    xml += 'flash-prebumper-refid="%s" ' % asset.get('refid')
        xml += '>\n'
        if self.short_description:
            xml += '<short-description><![CDATA[%(shortDescription)s]]>'
            xml += '</short-description>\n'
        if self.long_description:
            xml += '<long-description><![CDATA[%(longDescription)s]]>'
            xml += '</long-description>\n'
        for tag in self.tags:
            xml += '<tag><![CDATA[%s]]></tag>\n' % tag
        for asset in self.assets:
            if asset.get('encoding-rate', None):
                xml += '<rendition-refid>%s</rendition-refid>\n' % \
                    asset['refid']
        for meta in self.metadata:
            xml += '<custom-%s-value name="%s">%s</custom-%s-value>' % \
                (meta['type'], meta['key'], meta['value'], meta['type'])
        xml += '</title>'
        xml = xml % self._to_dict()
        return xml

    def _load(self, data):
        """
        Deserialize a dictionary of data into a ``pybrightcove.video.Video``
        object.
        """
        self.raw_data = data
        self.creation_date = _convert_tstamp(data['creationDate'])
        self.economics = data['economics']
        self.id = data['id']
        self.last_modified_date = _convert_tstamp(data['lastModifiedDate'])
        self.length = data['length']
        self.link_text = data['linkText']
        self.link_url = data['linkURL']
        self.long_description = data['longDescription']
        self.name = data['name']
        self.plays_total = data['playsTotal']
        self.plays_trailing_week = data['playsTrailingWeek']
        self.published_date = _convert_tstamp(data['publishedDate'])
        self.start_date = _convert_tstamp(data.get('startDate', None))
        self.end_date = _convert_tstamp(data.get('endDate', None))
        self.reference_id = data['referenceId']
        self.short_description = data['shortDescription']
        self.tags = []
        for tag in data['tags']:
            self.tags.append(tag)
        self.thumbnail_url = data['thumbnailURL']
        self.video_still_url = data['videoStillURL']

    def __setattr__(self, name, value):
        msg = None
        if value:
            if name == 'name' and len(value) > 255:
                msg = "Video.name must be 255 characters or less."
            if name == 'reference_id' and len(value) > 150:
                # val = value[:150]
                msg = "Video.reference_id must be 150 characters or less."
            if name == 'long_description' and len(value) > 5000:
                # val = value[:5000]
                msg = "Video.long_description must be 5000 characters or less."
            if name == 'short_description' and len(value) > 250:
                # val = value[:250]
                msg = "Video.short_description must be 250 characters or less."
            if name == 'item_state' and value not in (
                    enums.ItemStateEnum.ACTIVE,
                    enums.ItemStateEnum.INACTIVE):
                msg = "Video.item_state must be either ItemStateEnum.ACTIVE or"
                msg += " ItemStateEnum.INACTIVE"
            if name == 'video_full_length' and \
                    not isinstance(value, Rendition):
                msg = "Video.video_full_length must be of type Rendition"
            if name == 'economics' and value not in \
                    (enums.EconomicsEnum.FREE,
                     enums.EconomicsEnum.AD_SUPPORTED):
                msg = "Video.economics must be either EconomicsEnum.FREE or "
                msg += "EconomicsEnum.AD_SUPPORTED"

            if msg:
                raise exceptions.PyBrightcoveError(msg)
        return super(Video, self).__setattr__(name, value)
    
    def get_custom_metadata(self):
        """
        Fetches custom metadta for an already exisiting Video.
        """
        if self.id is not None:
            data = self.connection.get_item(
                'find_video_by_id',
                video_id=self.id,
                video_fields="customFields"
            )
            for key in data.get("customFields", {}).keys():
                val = data["customFields"].get(key)
                if val is not None:
                    self.add_custom_metadata(key, val)

    def add_custom_metadata(self, key, value, meta_type=None):
        """
        Add custom metadata to the Video.  meta_type is required for XML API.
        """
        self.metadata.append({'key': key, 'value': value, 'type': meta_type})

    def add_asset(self, filename, asset_type, display_name,
        encoding_rate=None, frame_width=None, frame_height=None,
        encode_to=None, encode_multiple=False,
        h264_preserve_as_rendition=False, h264_no_processing=False):
        """
        Add an asset to the Video object.
        """
        m = hashlib.md5()
        fp = file(filename, 'rb')
        bits = fp.read(262144)  ## 256KB
        while bits:
            m.update(bits)
            bits = fp.read(262144)
        fp.close()

        hash_code = m.hexdigest()
        refid = "%s-%s" % (os.path.basename(filename), hash_code)

        asset = {
            'filename': filename,
            'type': asset_type,
            'size': os.path.getsize(filename),
            'refid': refid,
            'hash-code': hash_code}

        if encoding_rate:
            asset.update({'encoding-rate': encoding_rate})
        if frame_width:
            asset.update({'frame-width': frame_width})
        if frame_height:
            asset.update({'frame-height': frame_height})
        if display_name:
            asset.update({'display-name': display_name})
        if encode_to:
            asset.update({'encode-to': encode_to})
            asset.update({'encode-multiple': encode_multiple})
            if encode_multiple and h264_preserve_as_rendition:
                asset.update({
                    'h264-preserve-as-rendition': h264_preserve_as_rendition})
        else:
            if h264_no_processing:
                asset.update({'h264-no-processing': h264_no_processing})
        self.assets.append(asset)

    def save(self, create_multiple_renditions=True,
        preserve_source_rendition=True,
        encode_to=enums.EncodeToEnum.FLV):
        """
        Creates or updates the video
        """
        if is_ftp_connection(self.connection) and len(self.assets) > 0:
            self.connection.post(xml=self.to_xml(), assets=self.assets)
        elif not self.id and self._filename:
            self.id = self.connection.post('create_video', self._filename,
                create_multiple_renditions=create_multiple_renditions,
                preserve_source_rendition=preserve_source_rendition,
                encode_to=encode_to,
                video=self._to_dict())
        elif not self.id and len(self.renditions) > 0:
            self.id = self.connection.post('create_video',
                video=self._to_dict())
        elif self.id:
            data = self.connection.post('update_video', video=self._to_dict())
            if data:
                self._load(data)

    def delete(self, cascade=False, delete_shares=False):
        """
        Deletes the video.
        """
        if self.id:
            self.connection.post('delete_video', video_id=self.id,
                cascade=cascade, delete_shares=delete_shares)
            self.id = None ## Prevent more activity on this video id

    def get_upload_status(self):
        """
        Get the status of the video that has been uploaded.
        """
        if self.id:
            return self.connection.post('get_upload_status', video_id=self.id)

    def share(self, accounts):
        """
        Create a share
        """
        if not isinstance(accounts, (list, tuple)):
            msg = "Video.share expects an iterable argument"
            raise exceptions.PyBrightcoveError(msg)
        raise exceptions.PyBrightcoveError("Not yet implemented")

    def set_image(self, image, filename=None, resize=False):
        """
        Set the poster or thumbnail of a this Vidoe.
        """
        if self.id:
            data = self.connection.post('add_image', filename,
                video_id=self.id, image=image.to_dict(), resize=resize)
            if data:
                self.image = Image(data=data)

    def find_related(self, _connection=None, page_size=100, page_number=0):
        """
        List all videos that are related to this one.
        """
        if self.id:
            return connection.ItemResultSet('find_related_videos',
                Video, _connection, page_size, page_number, None, None,
                video_id=self.id)

    def deactivate(self):
        """
        Mark a video as Inactive
        """
        self.item_state = enums.ItemStateEnum.INACTIVE
        self.save()

    @staticmethod
    def delete_video(video_id, cascade=False, delete_shares=False,
        _connection=None):
        """
        Delete the video represented by the ``video_id`` parameter.
        """
        c = _connection
        if not c:
            c = connection.APIConnection()
        c.post('delete_video', video_id=video_id, cascade=cascade,
            delete_shares=delete_shares)

    @staticmethod
    def get_status(video_id, _connection=None):
        """
        Get the status of a video given the ``video_id`` parameter.
        """
        c = _connection
        if not c:
            c = connection.APIConnection()
        return c.post('get_upload_status', video_id=video_id)

    @staticmethod
    def activate(video_id, _connection=None):
        """
        Mark a video as Active
        """
        c = _connection
        if not c:
            c = connection.APIConnection()
        data = c.post('update_video', video={
            'id': video_id,
            'itemState': enums.ItemStateEnum.ACTIVE})
        return Video(data=data, connection=c)

    @staticmethod
    def find_modified(since, filter_list=None, _connection=None, page_size=25,
        page_number=0, sort_by=enums.DEFAULT_SORT_BY,
        sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List all videos modified since a certain date.
        """
        filters = []
        if filter_list is not None:
            filters = filter_list
        if not isinstance(since, datetime):
            msg = 'The parameter "since" must be a datetime object.'
            raise exceptions.PyBrightcoveError(msg)
        fdate = int(since.strftime("%s")) / 60  ## Minutes since UNIX time
        return connection.ItemResultSet('find_modified_videos',
            Video, _connection, page_size, page_number, sort_by, sort_order,
            from_date=fdate, filter=filters)

    @staticmethod
    def find_all(_connection=None, page_size=100, page_number=0,
        sort_by=enums.DEFAULT_SORT_BY, sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List all videos.
        """
        return connection.ItemResultSet('find_all_videos', Video,
            _connection, page_size, page_number, sort_by, sort_order)

    @staticmethod
    def find_by_tags(and_tags=None, or_tags=None, _connection=None,
        page_size=100, page_number=0, sort_by=enums.DEFAULT_SORT_BY,
        sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List videos given a certain set of tags.
        """
        err = None
        if not and_tags and not or_tags:
            err = "You must supply at least one of either and_tags or or_tags."
        if and_tags and not isinstance(and_tags, (tuple, list)):
            err = "The and_tags argument for Video.find_by_tags must an "
            err += "iterable"
        if or_tags and not isinstance(or_tags, (tuple, list)):
            err = "The or_tags argument for Video.find_by_tags must an "
            err += "iterable"
        if err:
            raise exceptions.PyBrightcoveError(err)
        atags = None
        otags = None
        if and_tags:
            atags = ','.join([str(t) for t in and_tags])
        if or_tags:
            otags = ','.join([str(t) for t in or_tags])
        return connection.ItemResultSet('find_videos_by_tags',
            Video, _connection, page_size, page_number, sort_by, sort_order,
            and_tags=atags, or_tags=otags)

    @staticmethod
    def find_by_text(text, _connection=None, page_size=100, page_number=0,
        sort_by=enums.DEFAULT_SORT_BY, sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List videos that match the ``text`` in title or description.
        """
        return connection.ItemResultSet('find_videos_by_text',
            Video, _connection, page_size, page_number, sort_by, sort_order,
            text=text)

    @staticmethod
    def find_by_campaign(campaign_id, _connection=None, page_size=100,
        page_number=0, sort_by=enums.DEFAULT_SORT_BY,
        sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List all videos for a given campaign.
        """
        return connection.ItemResultSet(
            'find_videos_by_campaign_id', Video, _connection, page_size,
            page_number, sort_by, sort_order, campaign_id=campaign_id)

    @staticmethod
    def find_by_user(user_id, _connection=None, page_size=100, page_number=0,
        sort_by=enums.DEFAULT_SORT_BY, sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List all videos uploaded by a certain user.
        """
        return connection.ItemResultSet('find_videos_by_user_id',
            Video, _connection, page_size, page_number, sort_by, sort_order,
            user_id=user_id)

    @staticmethod
    def find_by_reference_ids(reference_ids, _connection=None, page_size=100,
        page_number=0, sort_by=enums.DEFAULT_SORT_BY,
        sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List all videos identified by a list of reference ids
        """
        if not isinstance(reference_ids, (list, tuple)):
            err = "Video.find_by_reference_ids expects an iterable argument"
            raise exceptions.PyBrightcoveError(err)
        ids = ','.join(reference_ids)
        return connection.ItemResultSet(
            'find_videos_by_reference_ids', Video, _connection, page_size,
            page_number, sort_by, sort_order, reference_ids=ids)

    @staticmethod
    def find_by_ids(ids, _connection=None, page_size=100, page_number=0,
        sort_by=enums.DEFAULT_SORT_BY, sort_order=enums.DEFAULT_SORT_ORDER):
        """
        List all videos identified by a list of Brightcove video ids
        """
        if not isinstance(ids, (list, tuple)):
            err = "Video.find_by_ids expects an iterable argument"
            raise exceptions.PyBrightcoveError(err)
        ids = ','.join([str(i) for i in ids])
        return connection.ItemResultSet('find_videos_by_ids',
            Video, _connection, page_size, page_number, sort_by, sort_order,
            video_ids=ids)




# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Test the Video object.
"""
import unittest
import uuid
from datetime import datetime, timedelta
import pybrightcove
import mock
## NOTE: This are ids private to my account, if you want to run these tests
##       for yourself and have them pass, replace these with your own values.
TEST_VIDEO_ID = 11449913001
TEST_VIDEO_IDS = [TEST_VIDEO_ID, 24780403001, 24780402001]
TEST_VIDEO_REF_ID = 'SN-47314834-100808-ATLGA-SV-404693da06d38.mp4'
TEST_VIDEO_REF_IDS = ['unittest-1', 'unittest-2', TEST_VIDEO_REF_ID]
VIDEO_DATA = {
    'creationDate': 1272312315.0,
    'economics': pybrightcove.enums.EconomicsEnum.FREE,
    'id': TEST_VIDEO_ID,
    'lastModifiedDate': 1272312315.0,
    'length': 55,
    'linkText': "the link text",
    'linkURL': "the link url",
    'longDescription': "A really long description.",
    'name': "My Video",
    'playsTotal': 100,
    'playsTrailingWeek': 40,
    'publishedDate': 1272312315.0,
    'startDate': 1272312315.0,
    'endDate': 1272312315.0,
    'referenceId': TEST_VIDEO_REF_ID,
    'shortDescription': "this is a short description",
    'tags': ['tag1', 'tag2', 'tag3'],
    'thumbnailURL': 'another_something_url',
    'videoStillURL': 'something_url'
}
IMAGE_DATA = {
    'id': 'image id',
    'referenceId': 'reference id',
    'type': 'image type',
    'displayName': 'display_name',
    'remoteUrl': 'image url'
}

class RenditionTest(unittest.TestCase):

    def test_invalid_size_type(self):
        try:
            r = pybrightcove.video.Rendition()
            r.remote_url = 'http://my.sample.com/flash.flv'
            r.size = 'invalid'
            r.video_duration = 600000
            r.video_codec = pybrightcove.enums.VideoCodecEnum.H264
            self.fail("Should have raised a PyBrightcoveError")
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEquals(str(e), "Rendition.size must be the number of bytes as an integer or long.")

    def test_invalid_duration_type(self):
        try:
            r = pybrightcove.video.Rendition()
            r.remote_url = 'http://my.sample.com/flash.flv'
            r.size = 10000000
            r.video_duration = "invalid"
            r.video_codec = pybrightcove.enums.VideoCodecEnum.H264
            self.fail("Should have raised a PyBrightcoveError")
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEquals(str(e), "Rendition.video_duration must be the duration in milliseconds as an integer or long.")

    def test_invalid_codec_type(self):
        try:
            r = pybrightcove.video.Rendition()
            r.remote_url = 'http://my.sample.com/flash.flv'
            r.size = 10000000
            r.video_duration = 600000
            r.video_codec = "INVALID"
            self.fail("Should have raised a PyBrightcoveError")
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEquals(str(e), "Rendition.video_codec must be SORENSON, ON2, or H264.")

    def test_serialization(self):
        r = pybrightcove.video.Rendition()
        r.remote_url = 'http://my.sample.com/flash.flv'
        r.size = 10000000
        r.video_duration = 600000
        r.video_codec = pybrightcove.enums.VideoCodecEnum.H264
        data = r.to_dict()
        self.assertTrue('remoteUrl' in data.keys())
        self.assertTrue('size' in data.keys())
        self.assertTrue('videoDuration' in data.keys())
        self.assertTrue('videoCodec' in data.keys())
        self.assertEquals(data['remoteUrl'], 'http://my.sample.com/flash.flv')
        self.assertEquals(data['size'], 10000000)
        self.assertEquals(data['videoDuration'], 600000)
        self.assertEquals(data['videoCodec'], pybrightcove.enums.VideoCodecEnum.H264)
        self.assertEquals(len(data.keys()), 4)


class VideoTest(unittest.TestCase):

    def setUp(self):
        self.test_uuid = str(uuid.uuid4())

    def _get_list_mock(self, ConnectionMock):
        m = ConnectionMock()
        c = mock.Mock()
        c.items = [mock.Mock(), mock.Mock()]
        c.total_count = 2
        c.page_size = 0
        m.get_list.return_value = c
        return m
    
    def _get_item_mock(self, ConnectionMock):
        m = ConnectionMock()
        m.get_item.return_value = {}
        return m

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_new(self, ConnectionMock):
        m = ConnectionMock()
        video = pybrightcove.video.Video(filename='/mnt/local/movie.mov', name='My Movie',
            short_description='This is my movie.')
        self.assertEquals(video.id, None)
        self.assertEquals(video.name, 'My Movie')
        self.assertEquals(video.short_description, 'This is my movie.')
        self.assertEquals(video.long_description, None)
        self.assertEquals(len(video.tags), 0)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_with_video_id(self, ConnectionMock):
        m = ConnectionMock()
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(id=TEST_VIDEO_ID)
        self.assertEquals(video.reference_id, TEST_VIDEO_REF_ID)
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[0][1][0], 'find_video_by_id')
        self.assertEquals(m.method_calls[0][2]['video_id'], TEST_VIDEO_ID)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_with_reference_id(self, ConnectionMock):
        m = ConnectionMock()
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(reference_id=TEST_VIDEO_REF_ID)
        self.assertEquals(video.id, TEST_VIDEO_ID)
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[0][1][0], 'find_video_by_reference_id')
        self.assertEquals(m.method_calls[0][2]['reference_id'], TEST_VIDEO_REF_ID)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_with_invalid_parameters(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name='This is wrong.')
            self.fail('Should not have worked, but rather raised an error.')
            video = pybrightcove.video.Video()
            self.fail('Should not have worked, but rather raised an error.')
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEquals(str(e), 'Invalid parameters for Video.')
        except Exception, e:
            self.fail('Should have thrown a pybrightcove.exceptions.PyBrightcoveError exception.')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_save_new(self, ConnectionMock):
        m = ConnectionMock()
        m.post.return_value = 10
        video = pybrightcove.video.Video(filename='bears.mov', name='The Bears',
            short_description='Opening roll for an exciting soccer match.')
        video.tags.append('unittest')
        self.assertEquals(video.id, None)
        video.save()
        self.assertEquals(video.id, 10)
        self.assertEquals(m.method_calls[0][0], 'post')
        self.assertEquals(m.method_calls[0][1][0], 'create_video')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_save_new_with_renditions(self, ConnectionMock):
        m = ConnectionMock()
        m.post.return_value = 10

        renditions = []
        r = pybrightcove.video.Rendition()
        r.remote_url = 'http://my.server.com/640_h264.flv'
        r.size = 232522522
        r.video_duration = 60000
        r.video_codec = pybrightcove.enums.VideoCodecEnum.H264
        renditions.append(r)

        r = pybrightcove.video.Rendition()
        r.remote_url = 'http://my.server.com/560_h264.flv'
        r.size = 23252252
        r.video_duration = 60000
        r.video_codec = pybrightcove.enums.VideoCodecEnum.H264
        renditions.append(r)
        
        r = pybrightcove.video.Rendition()
        r.remote_url = 'http://my.server.com/480_h264.flv'
        r.size = 2325225
        r.video_duration = 60000
        r.video_codec = pybrightcove.enums.VideoCodecEnum.H264
        renditions.append(r)

        video = pybrightcove.video.Video(name='The Bears', renditions=renditions,
            short_description='Opening roll for an exciting soccer match.')
        video.tags.append('unittest')
        
        self.assertEquals(video.id, None)
        video.save()
        m.post.return_value = {'id': 123456, 'referenceId': 777777, 'type': 'VIDEO_STILL', 'remoteUrl': 'http://my.sample.com/image-2', 'displayName': None}
        i = pybrightcove.video.Image()
        i.type = pybrightcove.enums.ImageTypeEnum.THUMBNAIL
        i.remote_url = 'http://my.sample.com/image-1.jpg'
        video.set_image(i)
        i = pybrightcove.video.Image()
        i.type = pybrightcove.enums.ImageTypeEnum.VIDEO_STILL
        i.remote_url = 'http://my.sample.com/image-2.jpg'
        video.set_image(i)
        
        self.assertEquals(video.id, 10)
        self.assertEquals(m.method_calls[0][0], 'post')
        self.assertEquals(m.method_calls[0][1][0], 'create_video')
        self.assertEquals(len(m.method_calls[0][2]['video']['renditions']), 3)
        self.assertEquals(m.method_calls[0][2]['video']['renditions'][1]['remoteUrl'], 'http://my.server.com/560_h264.flv')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'add_image')
        self.assertEquals(m.method_calls[1][2]['image']['remoteUrl'], 'http://my.sample.com/image-1.jpg')
        self.assertEquals(m.method_calls[2][0], 'post')
        self.assertEquals(m.method_calls[2][1][0], 'add_image')
        self.assertEquals(m.method_calls[2][2]['image']['remoteUrl'], 'http://my.sample.com/image-2.jpg')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_save_new_with_metadata(self, ConnectionMock):
        m = ConnectionMock()
        m.post.return_value = TEST_VIDEO_ID
        video = pybrightcove.video.Video(filename='bears.mov', name='The Bears',
            short_description='Opening roll for an exciting soccer match.')
        video.tags.append('unittest')
        self.assertEquals(video.id, None)
        video.add_custom_metadata('genre', 'Sci-Fi')
        video.add_custom_metadata('rating', 'PG-13')
        video.save()
        self.assertEquals(video.id, TEST_VIDEO_ID)
        self.assertEquals(m.method_calls[0][0], 'post')
        self.assertEquals(m.method_calls[0][1][0], 'create_video')
        self.assertTrue('unittest' in m.method_calls[0][2]['video']['tags'])
        self.assertTrue('customFields' in m.method_calls[0][2]['video'])
        self.assertTrue('genre' in m.method_calls[0][2]['video']['customFields'])
        self.assertTrue('rating' in m.method_calls[0][2]['video']['customFields'])
        self.assertEquals(m.method_calls[0][2]['video']['customFields']['genre'], 'Sci-Fi')
        self.assertEquals(m.method_calls[0][2]['video']['customFields']['rating'], 'PG-13')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_save_update(self, ConnectionMock):
        m = ConnectionMock()
        m.post.return_value = VIDEO_DATA
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(id=TEST_VIDEO_ID)
        video.tags.append('tag-%s' % self.test_uuid)
        video.tags.append('unittest')
        lmd = video.last_modified_date
        self.assertEquals(video.id, TEST_VIDEO_ID)
        video.save()
        self.assertEquals(video.id, TEST_VIDEO_ID)
        self.assertEquals(video.reference_id, TEST_VIDEO_REF_ID)
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'update_video')
        self.assertTrue('unittest' in m.method_calls[1][2]['video']['tags'])
        self.assertTrue('tag-%s' % self.test_uuid in m.method_calls[1][2]['video']['tags'])

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_save_update_with_metadata(self, ConnectionMock):
        m = ConnectionMock()
        m.post.return_value = VIDEO_DATA
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(id=TEST_VIDEO_ID)
        video.tags.append('tag-%s' % self.test_uuid)
        video.tags.append('unittest')
        video.add_custom_metadata('genre', 'Sci-Fi', 'string')
        video.add_custom_metadata('rating', 'PG-13', 'string')
        lmd = video.last_modified_date
        self.assertEquals(video.id, TEST_VIDEO_ID)
        video.save()
        self.assertEquals(video.id, TEST_VIDEO_ID)
        self.assertEquals(video.reference_id, TEST_VIDEO_REF_ID)
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'update_video')
        self.assertTrue('unittest' in m.method_calls[1][2]['video']['tags'])
        self.assertTrue('tag-%s' % self.test_uuid in m.method_calls[1][2]['video']['tags'])
        self.assertTrue('customFields' in m.method_calls[1][2]['video'])
        self.assertTrue('genre' in m.method_calls[1][2]['video']['customFields'])
        self.assertTrue('rating' in m.method_calls[1][2]['video']['customFields'])
        self.assertEquals(m.method_calls[1][2]['video']['customFields']['genre'], 'Sci-Fi')
        self.assertEquals(m.method_calls[1][2]['video']['customFields']['rating'], 'PG-13')


    @mock.patch('pybrightcove.connection.APIConnection')
    def test_get_upload_status(self, ConnectionMock):
        m = ConnectionMock()
        m.post.return_value = pybrightcove.enums.UploadStatusEnum.PROCESSING
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(id=TEST_VIDEO_ID)
        status = video.get_upload_status()
        self.assertEquals(status, pybrightcove.enums.UploadStatusEnum.PROCESSING)
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'get_upload_status')
        self.assertEquals(m.method_calls[1][2]['video_id'], TEST_VIDEO_ID)
        
    @mock.patch('pybrightcove.connection.APIConnection')
    def test_delete(self, ConnectionMock):
        m = ConnectionMock()
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(id=TEST_VIDEO_ID)
        video.delete()
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'delete_video')
        self.assertEquals(m.method_calls[1][2]['video_id'], TEST_VIDEO_ID)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_set_image(self, ConnectionMock):
        image = pybrightcove.video.Image(reference_id="img-%s" % self.test_uuid,
                      display_name="My Test Image",
                      type=pybrightcove.enums.ImageTypeEnum.VIDEO_STILL)
        m = ConnectionMock()
        m.post.return_value = IMAGE_DATA
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(id=TEST_VIDEO_ID)
        video.set_image(image, filename="IMG_0050.JPG")
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'add_image')
        self.assertEquals(m.method_calls[1][2]['video_id'], TEST_VIDEO_ID)
        self.assertEquals(video.image.to_dict(), IMAGE_DATA)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_tags(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        videos = pybrightcove.video.Video.find_by_tags(and_tags=['unittest', 'two'])
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_tags')
        self.assertEquals(m.method_calls[0][2]['and_tags'], ','.join(['unittest', 'two']))

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_text(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        videos = pybrightcove.video.Video.find_by_text('bear')
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_text')
        self.assertEquals(m.method_calls[0][2]['text'], 'bear')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_campaign(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        videos = pybrightcove.video.Video.find_by_campaign(988756758)
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_campaign_id')
        self.assertEquals(m.method_calls[0][2]['campaign_id'], 988756758)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_user(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        videos = pybrightcove.video.Video.find_by_user(12312431)
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_user_id')
        self.assertEquals(m.method_calls[0][2]['user_id'], 12312431)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_reference_ids(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        videos = pybrightcove.video.Video.find_by_reference_ids(TEST_VIDEO_REF_IDS)
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_reference_ids')
        self.assertEquals(m.method_calls[0][2]['reference_ids'], ','.join([str(x) for x in TEST_VIDEO_REF_IDS]))

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_ids(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        videos = pybrightcove.video.Video.find_by_ids(TEST_VIDEO_IDS)
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_ids')
        self.assertEquals(m.method_calls[0][2]['video_ids'], ','.join([str(x) for x in TEST_VIDEO_IDS]))
    
    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_id(self, ConnectionMock):
        m = self._get_item_mock(ConnectionMock)
        video = pybrightcove.video.Video(id=TEST_VIDEO_IDS[0])
        print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[0][1][0], 'find_video_by_id')
    
    @mock.patch('pybrightcove.connection.APIConnection')
    def test_get_metadata(self, ConnectionMock):
        m = self._get_item_mock(ConnectionMock)
        video = pybrightcove.video.Video(id=TEST_VIDEO_IDS[0])
        m.get_item.return_value = {"customFields": {"sample": "title"}}
        video.get_custom_metadata()
        print video #self.assertEquals(type(video), Video)
        print m.method_calls
        print video.metadata
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[0][1][0], 'find_video_by_id')
        self.assertEquals(m.method_calls[1][1][0], 'find_video_by_id')
        self.assertEquals(m.method_calls[1][2]["video_fields"], 'customFields')
        self.assertEquals(video.metadata[0]["key"], "sample")
        self.assertEquals(video.metadata[0]["value"], "title")

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_all(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        videos = pybrightcove.video.Video.find_all()
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_all_videos')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_related(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        m.get_item.return_value = VIDEO_DATA
        video = pybrightcove.video.Video(id=TEST_VIDEO_ID)
        for related_video in video.find_related():
            print related_video #self.assertEquals(type(related_video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[1][0], 'get_list')
        self.assertEquals(m.method_calls[1][1][0], 'find_related_videos')
        self.assertEquals(m.method_calls[1][2]['video_id'], TEST_VIDEO_ID)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_modified_unfiltered(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        yesterday = datetime.now() - timedelta(days=1)
        videos = pybrightcove.video.Video.find_modified(since=yesterday)
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_modified_videos')
        self.assertEquals(m.method_calls[0][2]['from_date'] > 0, True)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_modified_filtered(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        yesterday = datetime.now() - timedelta(days=1)
        filters = [pybrightcove.enums.FilterChoicesEnum.PLAYABLE,
            pybrightcove.enums.FilterChoicesEnum.DELETED]
        videos = pybrightcove.video.Video.find_modified(since=yesterday, filter_list=filters)
        for video in videos:
            print video #self.assertEquals(type(video), Video)
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_modified_videos')
        self.assertEquals(m.method_calls[0][2]['from_date'] > 0, True)
        self.assertEquals(m.method_calls[0][2]['filter'], filters)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_invalid_name(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name="Name is too long"*20,
                          short_description="ok desc",
                          filename="somefile.mov")
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEqual(str(e),
                "Video.name must be 255 characters or less.")
        else:
            self.fail("Expected pybrightcove.exceptions.PyBrightcoveError.")

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_invalid_short_description(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name="Name is too long",
                          short_description="ok desc"*100,
                          filename="somefile.mov")
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEqual(str(e),
                "Video.short_description must be 250 characters or less.")
        else:
            self.fail("Expected pybrightcove.exceptions.PyBrightcoveError.")

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_invalid_long_description(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name="Name is too long",
                          short_description="ok desc",
                          filename="somefile.mov")
            video.long_description = "Very long"*5000
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEqual(str(e),
                "Video.long_description must be 5000 characters or less.")
        else:
            self.fail("Expected pybrightcove.exceptions.PyBrightcoveError.")

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_invalid_reference_id(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name="Name is too long",
                          short_description="ok desc",
                          filename="somefile.mov")
            video.reference_id = "long ref id"*100
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEqual(str(e),
                "Video.reference_id must be 150 characters or less.")
        else:
            self.fail("Expected pybrightcove.exceptions.PyBrightcoveError.")

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_invalid_economics(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name="Name is too long",
                          short_description="ok desc",
                          filename="somefile.mov")
            video.economics = "The Keynesian view is Wrong"
        except pybrightcove.exceptions.PyBrightcoveError, e:
            err = "Video.economics must be either EconomicsEnum.FREE or"
            err += " EconomicsEnum.AD_SUPPORTED"
            self.assertEqual(str(e), err)
        else:
            self.fail("Expected pybrightcove.exceptions.PyBrightcoveError.")

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_invalid_item_state(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name="Name is too long",
                          short_description="ok desc",
                          filename="somefile.mov")
            video.item_state = "Invalid"
        except pybrightcove.exceptions.PyBrightcoveError, e:
            err = "Video.item_state must be either ItemStateEnum.ACTIVE or "
            err += "ItemStateEnum.INACTIVE"
            self.assertEqual(str(e), err)
        else:
            self.fail("Expected pybrightcove.exceptions.PyBrightcoveError.")

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_invalid_video_full_length(self, ConnectionMock):
        try:
            video = pybrightcove.video.Video(name="Name is too long",
                          short_description="ok desc",
                          filename="somefile.mov")
            video.video_full_length = 10
        except pybrightcove.exceptions.PyBrightcoveError, e:
            self.assertEqual(str(e),
                "Video.video_full_length must be of type Rendition")
        else:
            self.fail("Expected pybrightcove.exceptions.PyBrightcoveError.")



# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""
Configuration for pybrightcove is fairly simple and uses standard ini type file
structure found in ``~/.pybrightcove`` for a user level one or 
``/etc/pybrightcove.cfg`` for one at the system level.  User config files
supercede those at the system level.
"""

import os
import sys
import ConfigParser

from pybrightcove import __version__


USER_CONFIG_PATH = os.path.expanduser('~/.pybrightcove')
CONFIG_PATH = '/etc/pybrightcove.cfg'
CONFIG_LOCATIONS = [CONFIG_PATH, USER_CONFIG_PATH]
USER_AGENT = 'PyBrightcove/%s (%s)' % (__version__, sys.platform)


def has_option(section, name):
    """
    Wrapper around ConfigParser's ``has_option`` method.
    """
    cfg = ConfigParser.SafeConfigParser({"working_dir": "/tmp", "debug": "0"})
    cfg.read(CONFIG_LOCATIONS)
    return cfg.has_option(section, name)


def get(section, name):
    """
    Wrapper around ConfigParser's ``get`` method.
    """
    cfg = ConfigParser.SafeConfigParser({"working_dir": "/tmp", "debug": "0"})
    cfg.read(CONFIG_LOCATIONS)
    val = cfg.get(section, name)
    return val.strip("'").strip('"')




# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
The ``pybrightcove.playlist`` module supports all Brightcove functions dealing
with playlist objects.
"""

import pybrightcove
from pybrightcove.enums import DEFAULT_SORT_BY, DEFAULT_SORT_ORDER

VALID_PLAYLIST_TYPES = (pybrightcove.enums.PlaylistTypeEnum.EXPLICIT,
                        pybrightcove.enums.PlaylistTypeEnum.OLDEST_TO_NEWEST,
                        pybrightcove.enums.PlaylistTypeEnum.NEWEST_TO_OLDEST,
                        pybrightcove.enums.PlaylistTypeEnum.ALPHABETICAL,
                        pybrightcove.enums.PlaylistTypeEnum.PLAYS_TOTAL,
                        pybrightcove.enums.PlaylistTypeEnum.PLAYS_TRAILING_WEEK)
                        

class Playlist(object):
    """
    The Playlist object is a collection of Videos.
    """
    # pylint: disable=C0103,R0913,R0902
    # redefine type,id builtins - refactor later
    # pylint: disable=W0622
    def __init__(self, name=None, type=None, id=None, reference_id=None,
        data=None, connection=None):
        self.id = None
        self.reference_id = None
        self.account_id = None
        self.name = None
        self.short_description = None
        self.thumbnail_url = None
        self.videos = []
        self.video_ids = []
        self.type = None

        self.raw_data = None

        self.connection = connection
        if not self.connection:
            self.connection = pybrightcove.connection.APIConnection()

        if name and type in VALID_PLAYLIST_TYPES:
            self.name = name
            self.type = type
        elif id or reference_id:
            self.id = id
            self.reference_id = reference_id
            self._find_playlist()
        elif data:
            self._load(data)
        else:
            msg = "Invalid parameters for Playlist."
            raise pybrightcove.exceptions.PyBrightcoveError(msg)

    def __setattr__(self, name, value):
        msg = None
        if value:
            if name == 'name' and len(value) > 60:
                # val = value[:60] ## Is this better?
                msg = "Playlist.name must be 60 characters or less."
            if name == 'reference_id' and len(value) > 150:
                # val = value[:150]
                msg = "Playlist.reference_id must be 150 characters or less."
            if name == 'short_description' and len(value) > 250:
                # val = value[:250]
                msg = "Playlist.short_description must be 250 chars or less."
            if name == 'type' and value not in VALID_PLAYLIST_TYPES:
                msg = "Playlist.type must be a valid PlaylistTypeEnum"
            if msg:
                raise pybrightcove.exceptions.PyBrightcoveError(msg)
        return super(Playlist, self).__setattr__(name, value)

    def _find_playlist(self):
        """
        Internal method to populate the object given the ``id`` or
        ``reference_id`` that has been set in the constructor.
        """
        data = None
        if self.id:
            data = self.connection.get_item(
                'find_playlist_by_id', playlist_id=self.id)
        elif self.reference_id:
            data = self.connection.get_item(
                'find_playlist_by_reference_id',
                reference_id=self.reference_id)

        if data:
            self._load(data)

    def _to_dict(self):
        """
        Internal method that serializes object into a dictionary.
        """
        data = {
            'name': self.name,
            'referenceId': self.reference_id,
            'shortDescription': self.short_description,
            'playlistType': self.type,
            'id': self.id}
        if self.videos:
            for video in self.videos:
                if video.id not in self.video_ids:
                    self.video_ids.append(video.id)
        if self.video_ids:
            data['videoIds'] = self.video_ids
        [data.pop(key) for key in data.keys() if data[key] == None]
        return data

    def _load(self, data):
        """
        Internal method that deserializes a ``pybrightcove.playlist.Playlist``
        object.
        """
        self.raw_data = data
        self.id = data['id']
        self.reference_id = data['referenceId']
        self.name = data['name']
        self.short_description = data['shortDescription']
        self.thumbnail_url = data['thumbnailURL']
        self.videos = []
        self.video_ids = data['videoIds']
        self.type = data['playlistType']

        for video in data.get('videos', []):
            self.videos.append(pybrightcove.video.Video(
                data=video, connection=self.connection))

    def save(self):
        """
        Create or update a playlist.
        """
        d = self._to_dict()
        if len(d.get('videoIds', [])) > 0:
            if not self.id:
                self.id = self.connection.post('create_playlist', playlist=d)
            else:
                data = self.connection.post('update_playlist', playlist=d)
                if data:
                    self._load(data)

    def delete(self, cascade=False):
        """
        Deletes this playlist.
        """
        if self.id:
            self.connection.post('delete_playlist', playlist_id=self.id,
                cascade=cascade)
            self.id = None

    @staticmethod
    def find_all(connection=None, page_size=100, page_number=0,
        sort_by=DEFAULT_SORT_BY, sort_order=DEFAULT_SORT_ORDER):
        """
        List all playlists.
        """
        return pybrightcove.connection.ItemResultSet("find_all_playlists",
            Playlist, connection, page_size, page_number, sort_by, sort_order)

    @staticmethod
    def find_by_ids(ids, connection=None, page_size=100, page_number=0,
        sort_by=DEFAULT_SORT_BY, sort_order=DEFAULT_SORT_ORDER):
        """
        List playlists by specific IDs.
        """
        ids = ','.join([str(i) for i in ids])
        return pybrightcove.connection.ItemResultSet('find_playlists_by_ids',
            Playlist, connection, page_size, page_number, sort_by, sort_order,
            playlist_ids=ids)

    @staticmethod
    def find_by_reference_ids(reference_ids, connection=None, page_size=100,
        page_number=0, sort_by=DEFAULT_SORT_BY, sort_order=DEFAULT_SORT_ORDER):
        """
        List playlists by specific reference_ids.
        """
        reference_ids = ','.join([str(i) for i in reference_ids])
        return pybrightcove.connection.ItemResultSet(
            "find_playlists_by_reference_ids", Playlist, connection, page_size,
            page_number, sort_by, sort_order, reference_ids=reference_ids)

    @staticmethod
    def find_for_player_id(player_id, connection=None, page_size=100,
        page_number=0, sort_by=DEFAULT_SORT_BY, sort_order=DEFAULT_SORT_ORDER):
        """
        List playlists for a for given player id.
        """
        return pybrightcove.connection.ItemResultSet(
            "find_playlists_for_player_id", Playlist, connection, page_size,
            page_number, sort_by, sort_order, player_id=player_id)



# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Test the Playlist object
"""

import unittest
import uuid
import mock

from pybrightcove import enums
from pybrightcove import exceptions
from pybrightcove import playlist
from pybrightcove import video


TEST_VIDEO_ID = 11449913001
TEST_VIDEO_IDS = [TEST_VIDEO_ID, 24780403001, 24780402001]
TEST_PLAYLIST_ID = 24781161001
TEST_PLAYLIST_IDS = [TEST_PLAYLIST_ID, 10518202001]
TEST_PLAYLIST_REF_ID = 'unittest-playlist'
TEST_PLAYLIST_REF_IDS = [TEST_PLAYLIST_REF_ID, 'test']


class PlaylistTest(unittest.TestCase):

    def setUp(self):
        self.test_uuid = str(uuid.uuid4())

    def _get_list_mock(self, ConnectionMock):
        m = ConnectionMock()
        c = mock.Mock()
        c.items = [mock.Mock(), mock.Mock()]
        c.total_count = 2
        c.page_size = 0
        m.get_list.return_value = c
        return m

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_new(self, ConnectionMock):
        m = ConnectionMock()
        pl = playlist.Playlist(name='My Playlist', type=enums.PlaylistTypeEnum.EXPLICIT)
        pl.video_ids = TEST_VIDEO_IDS
        self.assertEquals(pl.id, None)
        self.assertEquals(pl.name, 'My Playlist')
        self.assertEquals(pl.type, enums.PlaylistTypeEnum.EXPLICIT)
        self.assertEquals(pl.video_ids, TEST_VIDEO_IDS)
        self.assertEquals(pl.short_description, None)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_with_playlist_id(self, ConnectionMock):
        m = ConnectionMock()
        m.get_item.return_value = {'id': TEST_PLAYLIST_ID, 'name': '', 'shortDescription': '', 'referenceId': TEST_PLAYLIST_REF_ID, 'thumbnailURL': '', 'videoIds': [], 'playlistType': ''}
        pl = playlist.Playlist(id=TEST_PLAYLIST_ID)
        self.assertEquals(pl.reference_id, TEST_PLAYLIST_REF_ID)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_with_reference_id(self, ConnectionMock):
        m = ConnectionMock()
        m.get_item.return_value = {'id': TEST_PLAYLIST_ID, 'name': '', 'shortDescription': '', 'referenceId': TEST_PLAYLIST_REF_ID, 'thumbnailURL': '', 'videoIds': [], 'playlistType': ''}
        pl = playlist.Playlist(reference_id=TEST_PLAYLIST_REF_ID)
        self.assertEquals(pl.id, TEST_PLAYLIST_ID)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_instantiate_with_invalid_parameters(self, ConnectionMock):
        try:
            pl = playlist.Playlist(name="No type specified")
            self.fail('Should have raised an error.')
        except exceptions.PyBrightcoveError, e:
            self.assertEquals(str(e), 'Invalid parameters for Playlist.')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_save_new(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        m.post.return_value = 10
        pl = playlist.Playlist(name="Unit Test Videos",
            type=enums.PlaylistTypeEnum.EXPLICIT)
        for v in video.Video.find_by_tags(and_tags=['unittest', ]):
            pl.videos.append(v)
        pl.save()
        self.assertEquals(pl.id, 10)
        self.assertEquals(pl.name, 'Unit Test Videos')
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_tags')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'create_playlist')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_save_update(self, ConnectionMock):
        m = ConnectionMock()
        data = {}
        data['id'] = TEST_PLAYLIST_ID
        data['referenceId'] = TEST_PLAYLIST_REF_ID
        data['name'] = "test-%s" % self.test_uuid
        data['shortDescription'] = "My description"
        data['thumbnailURL'] = "http://google.com"
        data['videoIds'] = TEST_VIDEO_IDS
        data['playlistType'] = enums.PlaylistTypeEnum.EXPLICIT
        m.get_item.return_value = data
        m.post.return_value = data
        pl = playlist.Playlist(id=TEST_PLAYLIST_ID)
        pl.name = 'test-%s' % self.test_uuid
        pl.save()
        self.assertEquals(pl.id, TEST_PLAYLIST_ID)
        self.assertEquals(pl.name, 'test-%s' % self.test_uuid)
        self.assertEquals(m.method_calls[0][0], 'get_item')
        self.assertEquals(m.method_calls[0][1][0], 'find_playlist_by_id')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'update_playlist')
        

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_delete(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        m.post.return_value = 10

        pl = playlist.Playlist(name="DELETE - Unit Test Videos",
            type=enums.PlaylistTypeEnum.EXPLICIT)
        for v in video.Video.find_by_tags(and_tags=['unittest', ]):
            pl.videos.append(v)
        self.assertEquals(pl.id, None)
        pl.save()
        self.assertEquals(pl.id, 10)
        pl.delete()
        self.assertEquals(pl.id, None)
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_videos_by_tags')
        self.assertEquals(m.method_calls[1][0], 'post')
        self.assertEquals(m.method_calls[1][1][0], 'create_playlist')
        self.assertEquals(m.method_calls[2][0], 'post')
        self.assertEquals(m.method_calls[2][1][0], 'delete_playlist')

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_ids(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        playlists = playlist.Playlist.find_by_ids(TEST_PLAYLIST_IDS)
        for pl in playlists:
            print pl
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_playlists_by_ids')
        self.assertEquals(m.method_calls[0][2]['playlist_ids'], ','.join([str(x) for x in TEST_PLAYLIST_IDS]))
        

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_by_reference_ids(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        playlists = playlist.Playlist.find_by_reference_ids(TEST_PLAYLIST_REF_IDS)
        for pl in playlists:
            print pl
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_playlists_by_reference_ids')
        self.assertEquals(m.method_calls[0][2]['reference_ids'], ','.join([str(x) for x in TEST_PLAYLIST_REF_IDS]))

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_for_player_id(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        playlists = playlist.Playlist.find_for_player_id(23424255)
        for pl in playlists:
            print pl
        print m.method_calls
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_playlists_for_player_id')
        self.assertEquals(m.method_calls[0][2]['player_id'], 23424255)

    @mock.patch('pybrightcove.connection.APIConnection')
    def test_find_all(self, ConnectionMock):
        m = self._get_list_mock(ConnectionMock)
        playlists = playlist.Playlist.find_all()
        for pl in playlists:
            print pl
        self.assertEquals(m.method_calls[0][0], 'get_list')
        self.assertEquals(m.method_calls[0][1][0], 'find_all_playlists')
        
        


# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# pylint: disable=C0111,R0903

class SortByType(object):
    """
    PUBLISH_DATE:
        Date title was published

    CREATION_DATE:
        Date title was created.

    MODIFIED_DATE
        Date title was last modified.

    PLAYS_TOTAL
        Number of times this title has been viewed.

    PLAYS_TRAILING_WEEK
        Number of times this title has been viewed in the past 7 days
        (excluding today).
    """
    PUBLISH_DATE = "PUBLISH_DATE"
    CREATION_DATE = "CREATION_DATE"
    MODIFIED_DATE = "MODIFIED_DATE"
    PLAYS_TOTAL = "PLAYS_TOTAL"
    PLAYS_TRAILING_WEEK = "PLAYS_TRAILING_WEEK"


class SortByOrderType(object):
    """
    ASC:
        Ascending

    DESC:
        Descending
    """
    ASC = "ASC"
    DESC = "DESC"


class UploadStatusEnum(object):
    """
    UPLOADING:
        File is still uploading

    PROCESSING:
        Upload complete; being processed.

    COMPLETE:
        Upload and processing complete.

    ERROR:
        Error in upload or processing.
    """
    UPLOADING = "UPLOADING"
    PROCESSING = "PROCESSING"
    COMPLETE = "COMPLETE"
    ERROR = "ERROR"


class EconomicsEnum(object):
    FREE = "FREE"
    AD_SUPPORTED = "AD_SUPPORTED"


class EncodeToEnum(object):
    MP4 = 'MP4'
    FLV = 'FLV'


class ItemStateEnum(object):
    ACTIVE = "ACTIVE"
    INACTIVE = "INACTIVE"
    DELETED = "DELETED"


class PlaylistTypeEnum(object):
    """
    EXPLICIT:
        A manual playlist, the videos of which were added individually.

    OLDEST_TO_NEWEST:
        A smart playlist, ordered from oldest to newest by last-modified date.

    NEWEST_TO_OLDEST:
        A smart playlist, ordered from newest to oldest by last-modified date.

    ALPHABETICAL:
        A smart playlist, ordered alphabetically.

    PLAYS_TOTAL:
        A smart playlist, ordered by total plays.

    PLAYS_TRAILING_WEEK:
        A smart playlist, ordered by most plays in the past week.
    """
    EXPLICIT = "EXPLICIT"
    OLDEST_TO_NEWEST = "OLDEST_TO_NEWEST"
    NEWEST_TO_OLDEST = "NEWEST_TO_OLDEST"
    ALPHABETICAL = "ALPHABETICAL"
    PLAYS_TOTAL = "PLAYS_TOTAL"
    PLAYS_TRAILING_WEEK = "PLAYS_TRAILING_WEEK"


class FilterChoicesEnum(object):
    PLAYABLE = "PLAYABLE"
    UNSCHEDULED = 'UNSCHEDULED'
    INACTIVE = 'INACTIVE'
    DELETED = 'DELETED'


class VideoCodecEnum(object):
    UNDEFINED = "UNDEFINED"
    NONE = "NONE"
    SORENSON = "SORENSON"
    ON2 = "ON2"
    H264 = "H264"


class ImageTypeEnum(object):
    VIDEO_STILL = "VIDEO_STILL"
    SYNDICATION_STILL = "SYNDICATION_STILL"
    THUMBNAIL = "THUMBNAIL"
    BACKGROUND = "BACKGROUND"
    LOGO = "LOGO"
    LOGO_OVERLAY = "LOGO_OVERLAY"


class VideoTypeEnum(object):
    FLV_PREVIEW = "FLV_PREVIEW"
    FLV_FULL = "FLV_FULL"
    FLV_BUMPER = "FLV_BUMPER"
    DIGITAL_MASTER = "DIGITAL_MASTER"


class AssetTypeEnum(object):
    VIDEO_FULL = "VIDEO_FULL"
    FLV_BUMPER = "FLV_BUMPER"
    THUMBNAIL = "THUMBNAIL"
    VIDEO_STILL = "VIDEO_STILL"
    BACKGROUND = "BACKGROUND"
    LOGO = "LOGO"
    LOGO_OVERLAY = "LOGO_OVERLAY"
    OTHER_IMAGE = "OTHER_IMAGE"


class CustomMetaType(object):
    ENUM = 'enum'
    STRING = 'string'


DEFAULT_SORT_BY = SortByType.CREATION_DATE
DEFAULT_SORT_ORDER = SortByOrderType.ASC



# pylint: disable=W,R,C
#!/usr/bin/env python
#
#    Copyright (C) 2009 Google Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


# This module is used for version 2 of the Google Data APIs.
# TODO: add proxy handling.


__author__ = 'j.s@google.com (Jeff Scudder)'


# Modifications made to this module by Patrick Altman <patrick@studionow.com>
# in order to support form-data Content Disposition so as to be compatible with
# Brightcove's Media API.
#
# Switched pybrightcove to use this library because the previously multipart
# module was reading the entire video file into memory before sending it on the
# wire, creating both an artifical limit to video file size as well as a very
# inefficient use of memory.


import os
import stat
import StringIO
import urlparse
import urllib
import httplib
import mimetypes


class Error(Exception):
  pass


class UnknownSize(Error):
  pass


class ProxyError(Error):
  pass


MIME_BOUNDARY = 'END_OF_PART'


class HttpRequest(object):
  """Contains all of the parameters for an HTTP 1.1 request.
 
  The HTTP headers are represented by a dictionary, and it is the
  responsibility of the user to ensure that duplicate field names are combined
  into one header value according to the rules in section 4.2 of RFC 2616.
  """
  method = None
  uri = None
 
  def __init__(self, uri=None, method=None, headers=None):
    """Construct an HTTP request.

    Args:
      uri: The full path or partial path as a Uri object or a string.
      method: The HTTP method for the request, examples include 'GET', 'POST',
              etc.
      headers: dict of strings The HTTP headers to include in the request.
    """
    self.headers = headers or {}
    self._body_parts = []
    if method is not None:
      self.method = method
    if isinstance(uri, (str, unicode)):
      uri = Uri.parse_uri(uri)
    self.uri = uri or Uri()
    self.headers['MIME-version'] = '1.0'
    self.headers['Connection'] = 'close'

  def add_body_part(self, key, data, mime_type, size=None):
    """Adds data to the HTTP request body.
   
    If more than one part is added, this is assumed to be a mime-multipart
    request. This method is designed to create MIME 1.0 requests as specified
    in RFC 1341.

    Args:
      data: str or a file-like object containing a part of the request body.
      mime_type: str The MIME type describing the data
      size: int Required if the data is a file like object. If the data is a
            string, the size is calculated so this parameter is ignored.
    """
    if isinstance(data, str):
      size = len(data)
    if hasattr(data, "fileno"):
      size = os.fstat(data.fileno())[stat.ST_SIZE]
    if size is None:
      # TODO: support chunked transfer if some of the body is of unknown size.
      raise UnknownSize('Each part of the body must have a known size.')
    if 'Content-Length' in self.headers:
      content_length = int(self.headers['Content-Length'])
    else:
      content_length = 0
    # If this is the first part added to the body, then this is not a multipart
    # request.
    boundary_string = '\r\n--%s\r\n' % (MIME_BOUNDARY,)
    self._body_parts.append(boundary_string)
    content_length += len(boundary_string) + size
    # Include the mime type of this part.
    cd = 'Content-Disposition: form-data; name="%s"' % key
    mt = mime_type
    if hasattr(data, "fileno"):
        cd += '; filename="%s"' % data.name.split('/')[-1]
        mt = mimetypes.guess_type(data.name)[0] or 'application/octet-stream'
    cd += '\r\n'
    type_string = 'Content-Type: %s\r\n\r\n' % (mt)
    self._body_parts.append(cd)
    self._body_parts.append(type_string)
    content_length += len(type_string) + len(cd)
    self._body_parts.append(data)
    self.headers['Content-Length'] = str(content_length)

  def end_of_parts(self):
    self._body_parts.append('\r\n--%s--' % (MIME_BOUNDARY,))
    content_length = int(self.headers['Content-Length'])
    content_length += len('\r\n--%s--' % (MIME_BOUNDARY,))
    self.headers['Content-Length'] = str(content_length)

  def _copy(self):
    """Creates a deep copy of this request."""
    copied_uri = Uri(self.uri.scheme, self.uri.host, self.uri.port,
                     self.uri.path, self.uri.query.copy())
    new_request = HttpRequest(uri=copied_uri, method=self.method,
                              headers=self.headers.copy())
    new_request._body_parts = self._body_parts[:]
    return new_request


def _apply_defaults(http_request):
  if http_request.uri.scheme is None:
    if http_request.uri.port == 443:
      http_request.uri.scheme = 'https'
    else:
      http_request.uri.scheme = 'http'


class Uri(object):
  """A URI as used in HTTP 1.1"""
  scheme = None
  host = None
  port = None
  path = None
 
  def __init__(self, scheme=None, host=None, port=None, path=None, query=None):
    """Constructor for a URI.

    Args:
      scheme: str This is usually 'http' or 'https'.
      host: str The host name or IP address of the desired server.
      post: int The server's port number.
      path: str The path of the resource following the host. This begins with
            a /, example: '/calendar/feeds/default/allcalendars/full'
      query: dict of strings The URL query parameters. The keys and values are
             both escaped so this dict should contain the unescaped values.
             For example {'my key': 'val', 'second': '!!!'} will become
             '?my+key=val&second=%21%21%21' which is appended to the path.
    """
    self.query = query or {}
    if scheme is not None:
      self.scheme = scheme
    if host is not None:
      self.host = host
    if port is not None:
      self.port = port
    if path:
      self.path = path
     
  def _get_query_string(self):
    param_pairs = []
    for key, value in self.query.iteritems():
      param_pairs.append('='.join((urllib.quote_plus(key),
          urllib.quote_plus(str(value)))))
    return '&'.join(param_pairs)

  def _get_relative_path(self):
    """Returns the path with the query parameters escaped and appended."""
    param_string = self._get_query_string()
    if self.path is None:
      path = '/'
    else:
      path = self.path
    if param_string:
      return '?'.join([path, param_string])
    else:
      return path
     
  def _to_string(self):
    if self.scheme is None and self.port == 443:
      scheme = 'https'
    elif self.scheme is None:
      scheme = 'http'
    else:
      scheme = self.scheme
    if self.path is None:
      path = '/'
    else:
      path = self.path
    if self.port is None:
      return '%s://%s%s' % (scheme, self.host, self._get_relative_path())
    else:
      return '%s://%s:%s%s' % (scheme, self.host, str(self.port),
                               self._get_relative_path())

  def __str__(self):
    return self._to_string()
     
  def modify_request(self, http_request=None):
    """Sets HTTP request components based on the URI."""
    if http_request is None:
      http_request = HttpRequest()
    if http_request.uri is None:
      http_request.uri = Uri()
    # Determine the correct scheme.
    if self.scheme:
      http_request.uri.scheme = self.scheme
    if self.port:
      http_request.uri.port = self.port
    if self.host:
      http_request.uri.host = self.host
    # Set the relative uri path
    if self.path:
      http_request.uri.path = self.path
    if self.query:
      http_request.uri.query = self.query.copy()
    return http_request

  ModifyRequest = modify_request

  def parse_uri(uri_string):
    """Creates a Uri object which corresponds to the URI string.
 
    This method can accept partial URIs, but it will leave missing
    members of the Uri unset.
    """
    parts = urlparse.urlparse(uri_string)
    uri = Uri()
    if parts[0]:
      uri.scheme = parts[0]
    if parts[1]:
      host_parts = parts[1].split(':')
      if host_parts[0]:
        uri.host = host_parts[0]
      if len(host_parts) > 1:
        uri.port = int(host_parts[1])
    if parts[2]:
      uri.path = parts[2]
    if parts[4]:
      param_pairs = parts[4].split('&')
      for pair in param_pairs:
        pair_parts = pair.split('=')
        if len(pair_parts) > 1:
          uri.query[urllib.unquote_plus(pair_parts[0])] = (
              urllib.unquote_plus(pair_parts[1]))
        elif len(pair_parts) == 1:
          uri.query[urllib.unquote_plus(pair_parts[0])] = None
    return uri

  parse_uri = staticmethod(parse_uri)

  ParseUri = parse_uri


parse_uri = Uri.parse_uri


ParseUri = Uri.parse_uri


class HttpResponse(object):
  status = None
  reason = None
  _body = None
 
  def __init__(self, status=None, reason=None, headers=None, body=None):
    self._headers = headers or {}
    if status is not None:
      self.status = status
    if reason is not None:
      self.reason = reason
    if body is not None:
      if hasattr(body, 'read'):
        self._body = body
      else:
        self._body = StringIO.StringIO(body)
         
  def getheader(self, name, default=None):
    if name in self._headers:
      return self._headers[name]
    else:
      return default

  def getheaders(self):
    return self._headers
   
  def read(self, amt=None):
    if self._body is None:
      return None
    if not amt:
      return self._body.read()
    else:
      return self._body.read(amt)


class HttpClient(object):
  """Performs HTTP requests using httplib."""
  debug = None
 
  def request(self, http_request):
    return self._http_request(http_request.method, http_request.uri, 
                              http_request.headers, http_request._body_parts)

  Request = request

  def _get_connection(self, uri, headers=None):
    """Opens a socket connection to the server to set up an HTTP request.
    
    Args:
      uri: The full URL for the request as a Uri object.
      headers: A dict of string pairs containing the HTTP headers for the
          request.
    """
    connection = None
    if uri.scheme == 'https':
      if not uri.port:
        connection = httplib.HTTPSConnection(uri.host)
      else:
        connection = httplib.HTTPSConnection(uri.host, int(uri.port))
    else:
      if not uri.port:
        connection = httplib.HTTPConnection(uri.host)
      else:
        connection = httplib.HTTPConnection(uri.host, int(uri.port))
    return connection

  def _http_request(self, method, uri, headers=None, body_parts=None):
    """Makes an HTTP request using httplib.
   
    Args:
      method: str example: 'GET', 'POST', 'PUT', 'DELETE', etc.
      uri: str or atom.http_core.Uri
      headers: dict of strings mapping to strings which will be sent as HTTP 
               headers in the request.
      body_parts: list of strings, objects with a read method, or objects
                  which can be converted to strings using str. Each of these
                  will be sent in order as the body of the HTTP request.
    """
    if isinstance(uri, (str, unicode)):
      uri = Uri.parse_uri(uri)
    connection = self._get_connection(uri, headers=headers)
    if self.debug:
      connection.debuglevel = 1

    if connection.host != uri.host:
      connection.putrequest(method, str(uri))
    else:
      connection.putrequest(method, uri._get_relative_path())

    # Overcome a bug in Python 2.4 and 2.5
    # httplib.HTTPConnection.putrequest adding
    # HTTP request header 'Host: www.google.com:443' instead of
    # 'Host: www.google.com', and thus resulting the error message
    # 'Token invalid - AuthSub token has wrong scope' in the HTTP response.
    if (uri.scheme == 'https' and int(uri.port or 443) == 443 and
        hasattr(connection, '_buffer') and
        isinstance(connection._buffer, list)):
      header_line = 'Host: %s:443' % uri.host
      replacement_header_line = 'Host: %s' % uri.host
      try:
        connection._buffer[connection._buffer.index(header_line)] = (
            replacement_header_line)
      except ValueError:  # header_line missing from connection._buffer
        pass

    # Send the HTTP headers.
    for header_name, value in headers.iteritems():
      connection.putheader(header_name, value)
    connection.endheaders()

    # If there is data, send it in the request.
    if body_parts:
      for part in body_parts:
        _send_data_part(part, connection)

    # Return the HTTP Response from the server.
    return connection.getresponse()

def _send_data_part(data, connection):
  if isinstance(data, (str, unicode)):
    # I might want to just allow str, not unicode.
    connection.send(data)
    return
  # Check to see if data is a file-like object that has a read method.
  elif hasattr(data, 'read'):
    # Read the file and send it a chunk at a time.
    while 1:
      binarydata = data.read(100000)
      if binarydata == '': break
      connection.send(binarydata)
    return
  else:
    # The data object was not a file.
    # Try to convert to a string and send the data.
    connection.send(str(data))
    return


class ProxiedHttpClient(HttpClient):

  def _get_connection(self, uri, headers=None):
    # Check to see if there are proxy settings required for this request.
    proxy = None
    if uri.scheme == 'https':
      proxy = os.environ.get('https_proxy')
    elif uri.scheme == 'http':
      proxy = os.environ.get('http_proxy')
    if not proxy:
      return HttpClient._get_connection(self, uri, headers=headers)
    # Now we have the URL of the appropriate proxy server.
    # Get a username and password for the proxy if required.
    proxy_auth = _get_proxy_auth()
    if uri.scheme == 'https':
      import socket
      if proxy_auth:
        proxy_auth = 'Proxy-authorization: %s' % proxy_auth
      # Construct the proxy connect command.
      port = uri.port
      if not port:
        port = 443
      proxy_connect = 'CONNECT %s:%s HTTP/1.0\r\n' % (uri.host, port)
      # Set the user agent to send to the proxy
      user_agent = ''
      if headers and 'User-Agent' in headers:
        user_agent = 'User-Agent: %s\r\n' % (headers['User-Agent'])
      proxy_pieces = '%s%s%s\r\n' % (proxy_connect, proxy_auth, user_agent)
      # Find the proxy host and port.
      proxy_uri = Uri.parse_uri(proxy)
      if not proxy_uri.port:
        proxy_uri.port = '80'
      # Connect to the proxy server, very simple recv and error checking
      p_sock = socket.socket(socket.AF_INET,socket.SOCK_STREAM)
      p_sock.connect((proxy_uri.host, int(proxy_uri.port)))
      p_sock.sendall(proxy_pieces)
      response = ''
      # Wait for the full response.
      while response.find("\r\n\r\n") == -1:
        response += p_sock.recv(8192)
      p_status = response.split()[1]
      if p_status != str(200):
        raise ProxyError('Error status=%s' % str(p_status))
      # Trivial setup for ssl socket.
      ssl = socket.ssl(p_sock, None, None)
      fake_sock = httplib.FakeSocket(p_sock, ssl)
      # Initalize httplib and replace with the proxy socket.
      connection = httplib.HTTPConnection(proxy_uri.host)
      connection.sock=fake_sock
      return connection
    elif uri.scheme == 'http':
      proxy_uri = Uri.parse_uri(proxy)
      if not proxy_uri.port:
        proxy_uri.port = '80'
      if proxy_auth:
        headers['Proxy-Authorization'] = proxy_auth.strip()
      return httplib.HTTPConnection(proxy_uri.host, int(proxy_uri.port))
    return None


def _get_proxy_auth():
  import base64
  proxy_username = os.environ.get('proxy-username')
  if not proxy_username:
    proxy_username = os.environ.get('proxy_username')
  proxy_password = os.environ.get('proxy-password')
  if not proxy_password:
    proxy_password = os.environ.get('proxy_password')
  if proxy_username:
    user_auth = base64.b64encode('%s:%s' % (proxy_username,
                                            proxy_password))
    return 'Basic %s\r\n' % (user_auth.strip())
  else:
    return ''



# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

# pylint: disable=C0111

class PyBrightcoveError(Exception):

    def __init__(self, reason):
        super(PyBrightcoveError, self).__init__(reason)


class ImproperlyConfiguredError(PyBrightcoveError):
    pass


class NoDataFoundError(PyBrightcoveError):

    def __init__(self, reason=None):
        super(NoDataFoundError, self).__init__(reason)


class BrightcoveError(Exception):
    description = "a general error"

    def __init__(self, description=None, raw_data=None):
        self.description = description or self.description
        self.raw_data = raw_data
        super(BrightcoveError, self).__init__(self.description, self.raw_data)

    @classmethod
    def raise_exception(cls, data):
        if "code" in data and data["code"] in ERROR_MAP:
            raise ERROR_MAP[data["code"]](data.get("message"), data)
        raise cls(raw_data=data)

    def __unicode__(self):
        return u'%s' % self.description

    def __str__(self):
        return self.description


class UnknownServerError(BrightcoveError):
    description = "an unknown error occurred while processing your request"
    cause = "An unspecified and unexpected error"
    code = 100


class ServiceDeployingError(BrightcoveError):
    description = """The write API is currently unavailable due to a scheduled
            upgrade. Please check http://studio.brightcove.com/ for updates."""
    cause = """The Brightcove Write API is currently down for a scheduled
            deployment."""
    code = 101


class CallTimeoutError(BrightcoveError):
    description = """The request you made is taking longer than expected to
            return. If requesting a large amount of data please try again with
            a smaller page_size."""
    cause = """Media API calls can time out before they complete if you are
            trying to get too much data at once. The page_size argument enables
            returning data in more manageable chunks."""
    code = 103


class EnctypeError(BrightcoveError):
    description = """wrong enctype; write methods require a multipart/form-data
            POST"""
    cause = "Calling a write method with a plain old POST"
    code = 200


class GetRequiredError(BrightcoveError):
    description = """methods that retrieve data must be called using GET
            requests"""
    cause = "Calling POST instead of GET on the Read API"
    code = 201


class PostRequiredError(BrightcoveError):
    description = """methods that change data must be called using POST
            requests"""
    cause = "Calling GET instead of POST on the Write API"
    code = 202


class MissingQueryStringError(BrightcoveError):
    description = "no query string provided"
    cause = "A missing query string in GET"
    code = 203


class MissingBodyError(BrightcoveError):
    description = """POST methods require valid JSON-RPC in the POST body,
            with \"method\" and \"params\" properties"""
    cause = "A missing parameter section in POST"
    code = 204


class MalformedParametersError(BrightcoveError):
    description = """unable to parse JSON-RPC parameters; they may be malformed
            or incorrect"""
    cause = "Malformed JSON in request"
    code = 205


class InvalidMethodError(BrightcoveError):
    description = "invalid method name"
    cause = "Invalid method name"
    code = 206


class FilestreamRequiredError(BrightcoveError):
    description = """upload requires a multipart/form-data POST with a valid
            filestream"""
    cause = "Missing file stream"
    code = 207


class MissingFileNameError(BrightcoveError):
    description = "file for upload must have a filename"
    cause = "Missing file name"
    code = 208


class UnwantedFilestreamError(BrightcoveError):
    description = "non-upload methods should not include a filestream"
    cause = "A filestream was provided for a non-upload method."
    code = 209


class InvalidTokenError(BrightcoveError):
    description = "invalid token"
    cause = "The token used in the call is invalid."
    code = 210


class MissingJSONError(BrightcoveError):
    description = "Could not find JSON-RPC."
    cause = """We got a null string for either the json parameter (for a
            non-multipart post) or the first part of a multipart post."""
    code = 211


class InvalidParametersError(BrightcoveError):
    description = "parameters are the wrong type or number for this method"
    cause = "Invoking the method threw an IllegalArgumentException"
    code = 301


class DeleteFailedError(BrightcoveError):
    description = """<message with reason for failure filled in by throwing
            method>"""
    cause = """Attempt to delete a business object failed. For example,
            attempts to delete a video that's in a manual playlist will
            fail."""
    code = 302


class RequiredParameterError(BrightcoveError):
    description = """required parameter <> was missing (<> to be filled in by
            throwing method)"""
    cause = "A required parameter was not set."
    code = 303


class IllegalValueError(BrightcoveError):
    description = """Cannot find any video with id: <>
            Cannot find the video you are trying to update: id <>
            One or more accounts in sharee_account_ids are not enabled for
                media sharing"""
    cause = """In a get_upload_status call, the video_id entered does not refer
                to a valid video in the Media Library
            In an update_video call, the video_id entered does not refer to a
                valid video in the Media Library
            In a share_video call, you do not have a sharing relationship
                established with the account whose id was entered."""
    code = 304


class IncompatibleValueError(BrightcoveError):
    description = """The geo restriction property on this video is incompatible
                with the destination account.
            The referenceId property on this video is in use by a destination
                account.
            The economics property on this video is incompatible with a
                destination account."""
    cause = """The Quova settings in the video and the destination account
                don't match.
            The reference ID is already in use in the destination account.
            Advertising must be enabled on all accounts, or disabled on all
                accounts."""
    code = 35


class FileFormatError(BrightcoveError):
    description = "The provided file was not in the expected format"
    cause = """The file provided to create_video was not in a supported file
            format."""
    code = 306


class ObjectNotFoundError(BrightcoveError):
    description = """The object specified by the given parameters could not be
            found."""
    cause = "The object with the specified id could not be found."
    code = 307


class NonmatchingChecksumError(BrightcoveError):
    description = """The provided file's MD5 digest did not match the checksum
            provided."""
    cause = """We checked an uploaded file's hash and it didn't match the
            expected checksum."""
    code = 308


class RemoteAssetsDisabledError(BrightcoveError):
    description = """This account is not approved to use remote assets."""
    cause = """You attempted to create or update a remote asset, but your
            account is not enabled for this feature."""
    code = 309


class InvalidCountryCodeError(BrightcoveError):
    description = """The following country codes are not supported for
        geo-restriction:"""
    cause = "You used an invalid country code for geo-restriction."
    code = 310


class GeoRestrictionDisabledError(BrightcoveError):
    description = "This account is not approved to use geo-restriction"
    cause = """You attempted to set geo-restriction properties for a video,
            but your account is not enabled for this feature."""
    code = 311


ERROR_MAP = {}
ERROR_MAP[100] = UnknownServerError
ERROR_MAP[101] = ServiceDeployingError
ERROR_MAP[103] = CallTimeoutError
ERROR_MAP[200] = EnctypeError
ERROR_MAP[201] = GetRequiredError
ERROR_MAP[202] = PostRequiredError
ERROR_MAP[203] = MissingQueryStringError
ERROR_MAP[204] = MissingBodyError
ERROR_MAP[205] = MalformedParametersError
ERROR_MAP[206] = InvalidMethodError
ERROR_MAP[207] = FilestreamRequiredError
ERROR_MAP[208] = MissingFileNameError
ERROR_MAP[209] = UnwantedFilestreamError
ERROR_MAP[210] = InvalidTokenError
ERROR_MAP[211] = MissingJSONError
ERROR_MAP[301] = InvalidParametersError
ERROR_MAP[302] = DeleteFailedError
ERROR_MAP[303] = RequiredParameterError
ERROR_MAP[304] = IllegalValueError
ERROR_MAP[305] = IncompatibleValueError
ERROR_MAP[306] = FileFormatError
ERROR_MAP[307] = ObjectNotFoundError
ERROR_MAP[308] = NonmatchingChecksumError
ERROR_MAP[309] = RemoteAssetsDisabledError
ERROR_MAP[310] = InvalidCountryCodeError
ERROR_MAP[311] = GeoRestrictionDisabledError



# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
"""
The connection objects found in this module facilite all the communication to
and from the actual APIs.
"""

import os
import hashlib
import simplejson
import urllib2
import urllib
import tempfile
import ftplib

from xml.dom import minidom

#import pybrightcove

from pybrightcove import config
from pybrightcove import http_core
from pybrightcove import enums
from pybrightcove import exceptions


class Connection(object):
    """
    Abstract base class for specific connection types.  This class should not
    be used directly in user code.
    """

    def _set(self, param, default=None, **kwargs):
        # pylint: disable=W,C,R
        if kwargs.get(param, None):
            setattr(self, param, kwargs[param])
        elif config.has_option('Connection', param):
            setattr(self, param, config.get('Connection', param))
        elif default:
            setattr(self, param, default)

    def __init__(self, **kwargs):
        # API Connection
        self._set('read_token', **kwargs)
        self._set('write_token', **kwargs)
        self._set('read_url',
            default='http://api.brightcove.com/services/library', **kwargs)
        self._set('write_url',
            default='http://api.brightcove.com/services/post', **kwargs)

        # FTP Connection
        self._set('host', default='upload.brightcove.com', **kwargs)
        self._set('user', **kwargs)
        self._set('password', **kwargs)
        self._set('publisher_id', **kwargs)
        self._set('preparer', **kwargs)
        self._set('report_success', **kwargs)

    def post(self, **kwargs):
        # pylint: disable=W,C,R
        raise Exception("Base class must implement this method.")

    def get_list(self, **kwargs):
        # pylint: disable=W,C,R
        raise Exception("Base class must implement this method.")

    def get_item(self, **kwargs):
        # pylint: disable=W,C,R
        raise Exception("Base class must implement this method.")


class FTPConnection(Connection):
    """
    Connection to use when wanting to interface with Brightcove via the batch
    FTP interface.
    """

    def __init__(self, host=None, user=None, password=None, publisher_id=None,
        preparer=None, report_success=False):
        # pylint: disable=R0913
        super(FTPConnection, self).__init__(host=host, user=user,
            password=password, publisher_id=publisher_id, preparer=preparer,
            report_success=report_success)
        self.notifications = []
        self.callback = None

    def get_manifest(self, asset_xml):
        """
        Construct and return the xml manifest to deliver along with video file.
        """
        # pylint: disable=E1101
        manifest = '<?xml version="1.0" encoding="utf-8"?>'
        manifest += '<publisher-upload-manifest publisher-id="%s" ' % \
            self.publisher_id
        manifest += 'preparer="%s" ' % self.preparer
        if self.report_success:
            manifest += 'report-success="TRUE">\n'
        for notify in self.notifications:
            manifest += '<notify email="%s"/>' % notify
        if self.callback:
            manifest += '<callback entity-url="%s"/>' % self.callback
        manifest += asset_xml
        manifest += '</publisher-upload-manifest>'
        return manifest

    def _send_file(self, filename):
        """
        Sends a file via FTP.
        """
        # pylint: disable=E1101
        ftp = ftplib.FTP(host=self.host)
        ftp.login(user=self.user, passwd=self.password)
        ftp.set_pasv(True)
        ftp.storbinary("STOR %s" % os.path.basename(filename),
            file(filename, 'rb'))

    def post(self, **kwargs):
        xml = kwargs.get("xml")
        assets = kwargs.get("assets")
        if xml is None or assets is None:
            raise Exception("Invalid keyword arguments!")

        ## Build manifest
        manifest = self.get_manifest(xml)

        ## Make sure it is well formed at least
        minidom.parseString(manifest)

        ## Record manifest
        fpno, fname = tempfile.mkstemp(suffix=".xml", 
            prefix="pybrightcove-manifest")
        fp_out = os.fdopen(fpno, 'wb')
        fp_out.write(manifest)
        fp_out.close()

        ## Upload files and manifest
        for asset in assets:
            self._send_file(asset['filename'])
        self._send_file(fname)

    def get_list(self, **kwargs):
        # pylint: disable=W,C,R
        raise Exception("This method is invalid for an FTP Connection")

    def get_item(self, **kwargs):
        # pylint: disable=W,C,R
        raise Exception("This method is invalid for an FTP Connection")


class APIConnection(Connection):
    """
    Connection to use when wanting to interface with the Brightcove Media API.
    """

    def __init__(self, read_token=None, write_token=None, read_url=None,
        write_url=None):
        super(APIConnection, self).__init__(read_token=read_token,
            write_token=write_token, read_url=read_url, write_url=write_url)
        if not hasattr(self, "read_token"):
            raise exceptions.ImproperlyConfiguredError(
                "Must specify at least a read_token.")

    def _post(self, data, file_to_upload=None):
        """
        Make the POST request.
        """
        # pylint: disable=E1101
        params = {"JSONRPC": simplejson.dumps(data)}
        req = None
        if file_to_upload:
            req = http_core.HttpRequest(self.write_url)
            req.method = 'POST'
            req.add_body_part("JSONRPC", simplejson.dumps(data), 'text/plain')
            upload = file(file_to_upload, "rb")
            req.add_body_part("filePath", upload, 'application/octet-stream')
            req.end_of_parts()
            content_type = "multipart/form-data; boundary=%s" % \
                http_core.MIME_BOUNDARY
            req.headers['Content-Type'] = content_type
            req.headers['User-Agent'] = config.USER_AGENT

            req = http_core.ProxiedHttpClient().request(req)
        else:
            msg = urllib.urlencode({'json': params['JSONRPC']})
            req = urllib2.urlopen(self.write_url, msg)

        if req:
            result = simplejson.loads(req.read())
            if 'error' in result and result['error']:
                exceptions.BrightcoveError.raise_exception(
                    result['error'])
            return result['result']

    def _get_response(self, **kwargs):
        """
        Make the GET request.
        """
        # pylint: disable=E1101
        url = self.read_url + "?output=JSON&token=%s" % self.read_token
        for key in kwargs:
            if key and kwargs[key]:
                val = kwargs[key]
                if isinstance(val, (list, tuple)):
                    val = ",".join(val)
                url += "&%s=%s" % (key, val)
        req = urllib2.urlopen(url)
        data = simplejson.loads(req.read())
        if data and data.get('error', None):
            exceptions.BrightcoveError.raise_exception(
                data['error'])
        if data == None:
            raise exceptions.NoDataFoundError(
                "No data found for %s" % repr(kwargs))
        return data

    def post(self, command, file_to_upload=None, **kwargs):
        # pylint: disable=W0221,E1101
        data = {"method": command}
        params = {"token": self.write_token}
        for key in kwargs:
            if key and kwargs[key]:
                params[key] = kwargs[key]
        if file_to_upload:
            md5 = hashlib.md5()
            file_handle = file(file_to_upload, 'rb')
            bits = file_handle.read(262144)  ## 256KB
            while bits:
                md5.update(bits)
                bits = file_handle.read(262144)
            file_handle.close()
            params['file_checksum'] = md5.hexdigest()
        data['params'] = params

        return self._post(data=data, file_to_upload=file_to_upload)

    def get_list(self, command, item_class, page_size, page_number, sort_by,
        sort_order, **kwargs):
        """
        Not intended to be called directly, but rather through an by the
        ItemResultSet object iterator.
        """
        # pylint: disable=R0913,W0221
        data = self._get_response(command=command,
                                  page_size=page_size,
                                  page_number=page_number,
                                  sort_by=sort_by,
                                  sort_order=sort_order,
                                  video_fields=None,
                                  get_item_count="true",
                                  **kwargs)
        return ItemCollection(data=data,
                              item_class=item_class,
                              connection=self)

    def get_item(self, command, **kwargs):
        # pylint: disable=W0221
        data = self._get_response(command=command, **kwargs)
        return data


def item_lister(command, connection, page_size, page_number, sort_by,
    sort_order, item_class, result_set, **kwargs):
    """
    A generator function for listing Video and Playlist objects.
    """
    # pylint: disable=R0913
    page = page_number
    while True:
        item_collection = connection.get_list(command,
                                             page_size=page_size,
                                             page_number=page,
                                             sort_by=sort_by,
                                             sort_order=sort_order,
                                             item_class=item_class,
                                             **kwargs)
        result_set.total_count = item_collection.total_count
        result_set.page_number = page
        for item in item_collection.items:
            yield item
        if item_collection.total_count < 0 or item_collection.page_size == 0:
            break
        if len(item_collection.items) > 0:
            page += 1
        else:
            break


class ItemResultSet(object):
    """
    An object to provide an interator facility to the paging calls to the API.
    """
    # pylint: disable=R0903,R0902

    def __init__(self, command, item_class, connection=None, page_size=100,
            page_number=0, sort_by=enums.DEFAULT_SORT_BY,
            sort_order=enums.DEFAULT_SORT_ORDER, **kwargs):
        # pylint: disable=R0913
        self.command = command
        if connection:
            self.connection = connection
        else:
            self.connection = APIConnection()
        self.page_size = page_size
        self.page_number = page_number
        self.sort_by = sort_by
        self.sort_order = sort_order
        self.item_class = item_class
        self.kwargs = kwargs
        self.total_count = None

    def __iter__(self):
        return item_lister(self.command, self.connection, self.page_size,
            self.page_number, self.sort_by, self.sort_order, self.item_class,
            self, **self.kwargs)


class ItemCollection(object):
    """
    The object that represents a collection of domain objects from the API.
    """
    # pylint: disable=R0903

    def __init__(self, data, item_class, connection=None):
        self.total_count = None
        self.items = None
        self.page_number = None
        self.page_size = None
        self.items = []
        self.data = data

        self.total_count = int(data['total_count'])
        self.page_number = int(data['page_number'])
        self.page_size = int(data['page_size'])
        for item in data['items']:
            self.items.append(item_class(data=item, connection=connection))



import os
from setuptools import setup, find_packages


VERSION = __import__("pybrightcove").__version__


setup(
    name="pybrightcove",
    version = VERSION,
    author="Patrick Altman",
    author_email="patrick@studionow.com",
    url="git://github.com/studionow/pybrightcove.git",
    description="""A Python interface for the Brightcove APIs""",
    packages=find_packages(),
    namespace_packages = [],
    include_package_data = True,
    zip_safe=False,
    license="MIT",
    install_requires=["simplejson",]
)





# Copyright (c) 2009 StudioNow, Inc <patrick@studionow.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from xml.dom import minidom
import unittest
import mock
from pybrightcove import video, enums, connection


class FTPVideoTest(unittest.TestCase):

    @mock.patch('ftplib.FTP')
    @mock.patch('hashlib.md5') # md5(), md5.hexdigest
    @mock.patch('os.path.getsize')
    @mock.patch('__builtin__.file')
    @mock.patch("os.fdopen")
    def test_batch_provision_video(self, FDOpenMockClass, OpenMockClass, GetSizeMockClass, Md5MockClass, FTPMockClass):
        fd = FDOpenMockClass()
        o = OpenMockClass()
        o.read.return_value = None
        m = Md5MockClass()
        m.hexdigest.return_value = 'a78fa9f8asd'
        GetSizeMockClass.return_value = 10000
        f = FTPMockClass()

        ftp = connection.FTPConnection(host='host',
                            user='user',
                            password='pass',
                            publisher_id='111111111',
                            preparer='Patrick',
                            report_success=True)
        v = video.Video(name="Some title",
                  reference_id='a532kallk3252a',
                  short_description="A short description.",
                  _connection=ftp)
        v.long_description = "An even longer description"
        v.tags.extend(["blah", "nah", "tag"])
        v.add_asset('1500.flv',
                enums.AssetTypeEnum.VIDEO_FULL, 'High quality rendition',
                encoding_rate=1500000, frame_width=640,
                frame_height=360)
        v.add_asset('700.flv',
                enums.AssetTypeEnum.VIDEO_FULL, 'Medium quality rendition',
                encoding_rate=700000, frame_width=640,
                frame_height=360)
        v.add_asset('poster.png',
                enums.AssetTypeEnum.VIDEO_STILL, 'Poster frame',
                frame_width=640, frame_height=360)
        v.save()

        self.assertEqual('login', f.method_calls[0][0])
        self.assertEqual('set_pasv', f.method_calls[1][0])
        self.assertEqual('storbinary', f.method_calls[2][0])
        self.assertEqual('STOR 1500.flv', f.method_calls[2][1][0])

        self.assertEqual('login', f.method_calls[3][0])
        self.assertEqual('set_pasv', f.method_calls[4][0])
        self.assertEqual('storbinary', f.method_calls[5][0])
        self.assertEqual('STOR 700.flv', f.method_calls[5][1][0])

        self.assertEqual('login', f.method_calls[6][0])
        self.assertEqual('set_pasv', f.method_calls[7][0])
        self.assertEqual('storbinary', f.method_calls[8][0])
        self.assertEqual('STOR poster.png', f.method_calls[8][1][0])

        self.assertEqual('write', fd.method_calls[2][0])
        valid_xml = minidom.parse(
            open('test_ftp_video_batch_provision_manifest.xml', 'rb'))
        test_xml = minidom.parseString(fd.method_calls[2][1][0])
        self.assertEqual(
            valid_xml.toxml().replace('\t', '').replace('\n', ''),
            test_xml.toxml().replace('\t', '').replace('\n', ''))

    @mock.patch('ftplib.FTP')
    @mock.patch('hashlib.md5')
    @mock.patch('os.path.getsize')
    @mock.patch('__builtin__.file')
    @mock.patch("os.fdopen")
    def test_batch_provision_with_custom_metadata_video(self, FDOpenMockClass, OpenMockClass, 
        GetSizeMockClass, Md5MockClass, FTPMockClass):
        fd = FDOpenMockClass()
        o = OpenMockClass()
        o.read.return_value = None
        m = Md5MockClass()
        m.hexdigest.return_value = 'a78fa9f8asd'
        GetSizeMockClass.return_value = 10000
        f = FTPMockClass()

        ftp = connection.FTPConnection(host='host',
                            user='user',
                            password='pass',
                            publisher_id='111111111',
                            preparer='Patrick',
                            report_success=True)
        v = video.Video(name="Some title",
                  reference_id='a532kallk3252a',
                  short_description="A short description.",
                  _connection=ftp)
        v.long_description = "An even longer description"
        v.tags.extend(["blah", "nah", "tag"])
        v.add_asset('1500.flv',
                enums.AssetTypeEnum.VIDEO_FULL, 'High quality rendition',
                encoding_rate=1500000, frame_width=640,
                frame_height=360)
        v.add_asset('700.flv',
                enums.AssetTypeEnum.VIDEO_FULL, 'Medium quality rendition',
                encoding_rate=700000, frame_width=640,
                frame_height=360)
        v.add_asset('poster.png',
                enums.AssetTypeEnum.VIDEO_STILL, 'Poster frame',
                frame_width=640, frame_height=360)

        v.add_custom_metadata("enum_one", "Value One", enums.CustomMetaType.ENUM)
        v.add_custom_metadata("enum_two", "Value Two", enums.CustomMetaType.ENUM)
        v.add_custom_metadata("key_one", "String Value One", enums.CustomMetaType.STRING)
        v.add_custom_metadata("key_two", "String Value Two", enums.CustomMetaType.STRING)
        v.save()

        self.assertEqual('login', f.method_calls[0][0])
        self.assertEqual('set_pasv', f.method_calls[1][0])
        self.assertEqual('storbinary', f.method_calls[2][0])
        self.assertEqual('STOR 1500.flv', f.method_calls[2][1][0])

        self.assertEqual('login', f.method_calls[3][0])
        self.assertEqual('set_pasv', f.method_calls[4][0])
        self.assertEqual('storbinary', f.method_calls[5][0])
        self.assertEqual('STOR 700.flv', f.method_calls[5][1][0])

        self.assertEqual('login', f.method_calls[6][0])
        self.assertEqual('set_pasv', f.method_calls[7][0])
        self.assertEqual('storbinary', f.method_calls[8][0])
        self.assertEqual('STOR poster.png', f.method_calls[8][1][0])

        self.assertEqual('write', fd.method_calls[0][0])
        valid_xml = minidom.parse(
            open('test_ftp_video_batch_provision_with_custom_metadata_manifest.xml', 'rb'))
        test_xml = minidom.parseString(fd.method_calls[0][1][0])
        self.assertEqual(
            valid_xml.toxml().replace('\t', '').replace('\n', ''),
            test_xml.toxml().replace('\t', '').replace('\n', ''))

